"""
Render documents as Reportlab PDF stories
"""
from __future__ import absolute_import

from cStringIO import StringIO
import cgi # For escape()

from pygov_br.vendor.pyth import document
from pygov_br.vendor.pyth.format import PythWriter

from reportlab.platypus import SimpleDocTemplate, Paragraph
from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
from reportlab.lib.pagesizes import A4
from reportlab.lib.units import inch

_tagNames = {'bold': 'b',
             'italic': 'i',
             'underline': 'u',
             'sub': 'sub',
             'super': 'super'}

LIST_INDENT = 0.3 * inch
BULLET_INDENT = 0.2 * inch
DEFAULT_PARA_SPACE = 0.2 * inch

BULLET_TEXT = "\xe2\x80\xa2"

class PDFWriter(PythWriter):

    @classmethod
    def write(klass, document, target=None, paragraphStyle=None):
        writer = PDFWriter(document, paragraphStyle)
        story = writer.go()

        if target is None:
            target = StringIO()

        doc = SimpleDocTemplate(target)
        doc.build(story)
        return target


    def __init__(self, doc, paragraphStyle=None):
        self.document = doc

        if paragraphStyle is None:
            stylesheet = getSampleStyleSheet()
            paragraphStyle = stylesheet['Normal']
        self.paragraphStyle = paragraphStyle
        self.paragraphStyle.spaceAfter = 0.2 * inch

        self.paragraphDispatch = {
            document.List: self._list,
            document.Paragraph: self._paragraph}


    def go(self):
        self.paragraphs = []
        for para in self.document.content:
            self._dispatch(para)
        return self.paragraphs


    def _dispatch(self, para, level=0, **kw):
        handler = self.paragraphDispatch[type(para)]
        return handler(para, level=level, **kw)


    def _paragraph(self, paragraph, level=0, bulletText=None):
        text = u"".join(self._text(t) for t in paragraph.content)
        self.paragraphs.append(Paragraph(text, self.paragraphStyle, bulletText=bulletText))


    def _text(self, text):
        content = cgi.escape(u"".join(text.content))

        tags = []
        for prop, value in list(text.properties.items()):
            if prop == "url":
                tags.append((u'<u><link destination="%s" color="blue">' % value, u"</link></u>"))
            if prop in _tagNames:
                tag = _tagNames[prop]
                tags.append((u"<%s>" % tag, u"</%s>" % tag))

        open_tags = u"".join(tag[0] for tag in tags)
        close_tags = u"".join(tag[1] for tag in reversed(tags))
        return u"%s%s%s" % (open_tags, content, close_tags)


    def _list(self, plist, level=0, bulletText=None):
        for entry in plist.content:
            self._list_entry(entry, level=level+1)


    def _list_entry(self, entry, level):
        first = True
        prevStyle = self.paragraphStyle

        self.paragraphStyle = ParagraphStyle("ListStyle", self.paragraphStyle)

        for para in entry.content:

            if first:
                bullet = BULLET_TEXT
                self.paragraphStyle.leftIndent = LIST_INDENT * level
                self.paragraphStyle.bulletIndent = (LIST_INDENT * level - 1) + BULLET_INDENT
            else:
                bullet = None
                self.paragraphStyle.leftIndent = LIST_INDENT * (level + 1)

            self._dispatch(para, level=level, bulletText=bullet)

            first = False

        self.paragraphStyle = prevStyle






from pygov_br.django_apps.base import BaseDataImporter
from pygov_br.django_apps.camara_deputados import models
from pygov_br.camara_deputados import cd


class LegislativeBodyTypeImporter(BaseDataImporter):

    field_relation = {
        'id': 'id',
        'description': 'descricao'
    }

    def get_model(self):
        return models.LegislativeBodyType

    def get_data(self):
        return cd.legislative_bodies.types()


class LegislativeBodyRoleImporter(BaseDataImporter):

    field_relation = {
        'id': 'id',
        'description': 'descricao'
    }

    def get_model(self):
        return models.LegislativeBodyRole

    def get_data(self):
        return cd.legislative_bodies.roles()


class LegislativeBodyImporter(BaseDataImporter):

    field_relation = {
        'id': 'id',
        'legislative_body_type': 'idTipodeOrgao',
        'initials': 'sigla',
        'description': 'descricao',
    }

    roles = {
        'Presidente': 'Presidente',
        'PrimeiroVice-Presidente': '1º Vice-Presidente',
        'SegundoVice-Presidente': '2º Vice-Presidente',
        'TerceiroVice-Presidente': '3º Vice-Presidente',
        'Relator': 'Relator',
        'Relator-Parcial': 'Relator-Parcial',
        'PrimeiroSecretário': '1º Secretário',
        'SegundoSecretário': '2º Secretário',
        'TerceiroSecretário': '3º Secretário',
        'QuartoSecretário': '4º Secretário',
        'PrimeiroSuplente-de-Secretário': '1º Suplente de Secretário',
        'SegundoSuplente-de-Secretário': '2º Suplente de Secretário',
        'TerceiroSuplente-de-Secretário': '3º Suplente de Secretário',
        'QuartoSuplente-de-Secretário': '4º Suplente de Secretário',
    }

    def get_model(self):
        return models.LegislativeBody

    def get_data(self):
        return cd.legislative_bodies.all()

    def clean_legislative_body_type(self, data):
        return models.LegislativeBodyType.objects.get(id=data)

    def after_save_object(self, obj):
        if obj.initials == 'PLEN':
            return

        members = cd.legislative_bodies.members(obj.id)
        if members is not None:
            for role in members.keys():
                if role == 'membro':
                    self._save_common_members(members, obj)
                else:
                    self._save_special_members(members[role], role, obj)

    def _save_special_members(self, members, role, legislative_body):
        if isinstance(members, list):
            for member in members:
                self._save_member(member['ideCadastro'], role=role,
                                  legislative_body=legislative_body)
        else:
            self._save_member(members['ideCadastro'], role=role,
                              legislative_body=legislative_body)

    def _save_common_members(self, members, legislative_body):
        for member in members['membro']:
            self._save_member(member['ideCadastro'], legislative_body)

    def _save_member(self, deputy_id, legislative_body, role=False):
        deputy = self._get_deputy(deputy_id)
        obj_dict = {}
        if deputy is not None:
            obj_dict['deputy'] = deputy
            obj_dict['legislative_body'] = legislative_body

            if role:
                obj_dict['role'] = models.LegislativeBodyRole.objects.get(
                    description=self.roles[role]
                )

        if obj_dict:
            models.LegislativeBodyMember.objects.get_or_create(**obj_dict)

    def _get_deputy(self, deputy_id):
        try:
            return models.Deputy.objects.get(pk=deputy_id)
        except models.Deputy.DoesNotExist:
            return None



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-07-19 23:30
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('camara_deputados', '0004_auto_20170719_2314'),
    ]

    operations = [
        migrations.AddField(
            model_name='proposal',
            name='indexes',
            field=models.CharField(blank=True, max_length=500, null=True),
        ),
        migrations.AddField(
            model_name='proposal',
            name='link',
            field=models.CharField(blank=True, max_length=500, null=True),
        ),
    ]



from django.db import models


class LegislativeBodyType(models.Model):

    class Meta:
        verbose_name = "Legislative Body Type"
        verbose_name_plural = "Legislative Body Types"

    def __str__(self):
        return self.description

    description = models.CharField(max_length=100)


class LegislativeBody(models.Model):

    class Meta:
        verbose_name = "Legislative Body"
        verbose_name_plural = "Legislative Bodies"

    def __str__(self):
        return self.initials

    legislative_body_type = models.ForeignKey('LegislativeBodyType')
    initials = models.CharField(max_length=10)
    description = models.TextField()
    members = models.ManyToManyField('Deputy', through='LegislativeBodyMember')


class LegislativeBodyMember(models.Model):

    class Meta:
        verbose_name = "Legislative Body Member"
        verbose_name_plural = "Legislative Body Members"

    def __str__(self):
        return self.legislative_body.initials + ' - ' + self.deputy.name

    legislative_body = models.ForeignKey('LegislativeBody')
    deputy = models.ForeignKey('Deputy', related_name='comissions')
    role = models.ForeignKey('LegislativeBodyRole', null=True)


class LegislativeBodyRole(models.Model):

    class Meta:
        verbose_name = "Legislative Body Role"
        verbose_name_plural = "Legislative Body Roles"

    def __str__(self):
        return self.description

    description = models.CharField(max_length=100)


class Party(models.Model):

    class Meta:
        verbose_name = "Party"
        verbose_name_plural = "Parties"

    def __str__(self):
        return self.name

    creation_date = models.DateField(null=True)
    extinction_date = models.DateField(null=True)
    name = models.CharField(max_length=100)
    initials = models.CharField(max_length=20, primary_key=True)


class PartyBloc(models.Model):

    class Meta:
        verbose_name = "Party Bloc"
        verbose_name_plural = "Party Blocs"

    def __str__(self):
        return self.name

    creation_date = models.DateField(null=True)
    extinction_date = models.DateField(null=True)
    name = models.CharField(max_length=100)
    initials = models.CharField(max_length=50)
    members = models.ManyToManyField('Party', through='PartyBlocMember')


class PartyBlocMember(models.Model):

    class Meta:
        verbose_name = "Party Bloc Member"
        verbose_name_plural = "Party Bloc Members"

    def __str__(self):
        return self.party_bloc.name + ' - ' + self.party.initials

    party_bloc = models.ForeignKey('PartyBloc')
    party = models.ForeignKey('Party', related_name='party_blocs')
    adhesion_date = models.DateField(null=True)
    shutdown_date = models.DateField(null=True)


class ParliamentarySeat(models.Model):

    class Meta:
        verbose_name = "Parliamentary Seat"
        verbose_name_plural = "Parliamentary Seats"

    def __str__(self):
        return self.name

    name = models.CharField(max_length=150)
    initials = models.CharField(max_length=20, primary_key=True)


class Deputy(models.Model):

    class Meta:
        verbose_name = "Deputy"
        verbose_name_plural = "Deputies"

    def __str__(self):
        return self.parliamentary_name

    outbuilding = models.IntegerField()
    budget_id = models.IntegerField(null=True)
    condition = models.CharField(max_length=50)
    email = models.EmailField()
    phone = models.CharField(max_length=15)
    cabinet = models.IntegerField()
    parliamentary_id = models.IntegerField()
    enrollment_id = models.IntegerField()
    name = models.CharField(max_length=150)
    parliamentary_name = models.CharField(max_length=100)
    party = models.ForeignKey('Party', related_name='members')
    gender = models.CharField(max_length=50)
    region = models.CharField(max_length=2)
    photo_url = models.URLField()
    parliamentary_seat = models.ForeignKey('ParliamentarySeat', null=True,
                                           related_name='members')
    parliamentary_seat_leader = models.BooleanField(default=False)


class SessionType(models.Model):

    class Meta:
        verbose_name = "SessionType"
        verbose_name_plural = "Session Types"

    def __str__(self):
        return self.description

    description = models.CharField(max_length=255)


class Session(models.Model):

    class Meta:
        verbose_name = "Session"
        verbose_name_plural = "Sessions"

    def __str__(self):
        return self.code + ' - ' + self.session_type.description

    code = models.CharField(primary_key=True, max_length=255)
    date = models.DateField()
    session_type = models.ForeignKey('SessionType')
    number = models.IntegerField()


class SessionPhase(models.Model):

    class Meta:
        verbose_name = "Session Phase"
        verbose_name_plural = "Session Phases"

    def __str__(self):
        return self.description

    code = models.CharField(primary_key=True, max_length=50)
    description = models.CharField(max_length=50)


class Speech(models.Model):

    class Meta:
        verbose_name = "Speech"
        verbose_name_plural = "Speeches"

    def __str__(self):
        return str(self.author) + ' - ' + str(self.initial_time)

    initial_time = models.DateTimeField()
    insertion_number = models.IntegerField()
    quarter_number = models.IntegerField()
    order = models.IntegerField()
    full_text = models.TextField()
    summary = models.TextField(null=True)
    indexes = models.TextField(null=True)
    author = models.ForeignKey('Deputy', related_name='speeches', null=True)
    session = models.ForeignKey('Session', related_name='speeches')
    session_phase = models.ForeignKey('SessionPhase', related_name='speeches')

    @property
    def text_url(self):
        url = 'http://www.camara.leg.br/internet/sitaqweb/TextoHTML.asp?' \
              'etapa={}&nuSessao={}&nuQuarto={}&nuOrador={}&nuInsercao={}&' \
              'dtHorarioQuarto={}'
        return url.format(
            self.session.number,
            self.session.code,
            self.quarter_number,
            self.order,
            self.insertion_number,
            self.initial_time.strftime('%H:%M')
        )

    def save(self, *args, **kwargs):
        if self.author is not None:
            super(Speech, self).save(*args, **kwargs)


class ProposalType(models.Model):

    class Meta:
        verbose_name = "Proposal Type"
        verbose_name_plural = "Proposal Types"

    is_active = models.BooleanField()
    description = models.CharField(max_length=500)
    gender = models.CharField(max_length=2)
    initials = models.CharField(max_length=50)

    def __str__(self):
        return self.description


class Proposal(models.Model):

    class Meta:
        verbose_name = "Proposal"
        verbose_name_plural = "Proposals"

    year = models.IntegerField()
    number = models.IntegerField()
    author = models.ForeignKey(Deputy, blank=True, null=True,
                               related_name='proposals')
    submission_date = models.DateField()
    name = models.CharField(max_length=500)
    summary = models.TextField(blank=True, null=True)
    summary_explanation = models.TextField(blank=True, null=True)
    proposal_type = models.ForeignKey(ProposalType)
    indexes = models.CharField(max_length=500, blank=True, null=True)
    link = models.CharField(max_length=500, blank=True, null=True)



from pygov_br.base import ClientWrapper
from .deputy import DeputyClient
from .legislative_body import LegislativeBodyClient
from .proposal import ProposalClient
from .session import SessionClient

cd = ClientWrapper(
    deputies=DeputyClient,
    legislative_bodies=LegislativeBodyClient,
    proposals=ProposalClient,
    sessions=SessionClient
)



# -*- coding: utf-8 -*-
from django.db import transaction
import abc
import click


class BaseDataImporter(object):

    def __new__(cls):
        for field in cls.field_relation.keys():
            def _method(self, data):
                return data
            if not hasattr(cls, 'clean_' + field):
                setattr(cls, 'clean_' + field, _method)
        return super(BaseDataImporter, cls).__new__(cls)

    @abc.abstractmethod
    def get_data(self):
        return

    @abc.abstractmethod
    def get_model(self):
        return

    def get_progressbar_label(self):
        return 'Importing data'

    def after_save_object(self, obj):
        pass

    def save_data(self):
        print('Downloading data...')
        data = self.get_data()
        if isinstance(data, dict):
            self._get_object(data)
        elif isinstance(data, list):
            with click.progressbar(data,
                                   label=self.get_progressbar_label()) as data:
                for data_row in data:
                    self._get_object(data_row)

    def _get_object(self, data):
        obj = self._fill_model(self.get_model(), data)
        self.after_save_object(obj)
        return obj

    @transaction.atomic
    def _fill_model(self, model_class, data):
        obj_dict = {}

        for field in self.field_relation.keys():
            try:
                clean_method = getattr(self, 'clean_' + field)
                key = self.field_relation[field]
                cleaned_field = clean_method(data.get(key, None))

                if cleaned_field == '':
                    cleaned_field = None

                obj_dict[field] = cleaned_field
            except KeyError:
                continue
            except AttributeError:
                continue
        obj, created = model_class.objects.get_or_create(**obj_dict)
        return obj



"""
Render documents as latex.

For the moment we generate the latex document from the
reStructuredText writer output.
"""
from __future__ import absolute_import

from cStringIO import StringIO
import docutils.core

from pygov_br.vendor.pyth import document
from pygov_br.vendor.pyth.format import PythWriter
from pygov_br.vendor.pyth.plugins.rst.writer import RSTWriter


class LatexWriter(PythWriter):

    @classmethod
    def write(klass, document, target=None, stylesheet=""):
        """
        convert a pyth document to a latex document

        we can specify a stylesheet as a latex document fragment that
        will be inserted after the headers.  This way we can override
        the default style.
        """
        writer = LatexWriter(document, target, stylesheet)
        return writer.go()

    def __init__(self, doc, target=None, stylesheet=""):
        """Create a writer that produce a latex document

        we can specify a stylesheet as a latex document fragment that
        will be inserted after the headers.  This way we can override
        the default style.
        """
        self.document = doc
        self.stylesheet = stylesheet
        self.target = target if target is not None else StringIO()

    @property
    def full_stylesheet(self):
        """
        Return the style sheet that will ultimately be inserted into
        the latex document.

        This is the user given style sheet plus some additional parts
        to add the meta data.
        """
        latex_fragment = r"""
        \usepackage[colorlinks=true,linkcolor=blue,urlcolor=blue]{hyperref}
        \hypersetup{
           pdftitle={%s},
           pdfauthor={%s},
           pdfsubject={%s}
        }
        """ % (self.document.properties.get("title"),
               self.document.properties.get("author"),
               self.document.properties.get("subject"))
        return latex_fragment + self.stylesheet

    def go(self):
        rst = RSTWriter.write(self.document).getvalue()
        settings = dict(input_encoding="UTF-8",
                        output_encoding="UTF-8",
                        stylesheet="stylesheet.tex")
        latex = docutils.core.publish_string(rst,
                                             writer_name="latex",
                                             settings_overrides=settings)
        # We don't want to keep an \input command in the latex file
        latex = latex.replace(r"\input{stylesheet.tex}",
                              self.full_stylesheet)
        self.target.write(latex)
        return self.target



"""
Render documents as RTF 1.5

http://www.biblioscape.com/rtf15_spec.htm
"""
from __future__ import absolute_import

from pygov_br.vendor.pyth import document
from pygov_br.vendor.pyth.format import PythWriter

from cStringIO import StringIO
import six
from six.moves import range


# XXX Todo -- Make these parameters
PARAGRAPH_SPACING = 150
LIST_ITEM_SPACING = 50

_styleFlags = {
    'bold': r'\b',
    'italic': r'\i',
    'underline': r'\ul',
}


class Rtf15Writer(PythWriter):

    # Calibri is the default font in Office2007.
    # So we'll use that for swiss, and let it fall back
    # to Arial everywhere else.
    fonts = {
        'swiss': 'Calibri',
        'roman': 'Times New Roman',
    }


    @classmethod
    def write(klass, document, target=None, fontFamily='roman'):
        if target is None:
            target = StringIO()

        writer = Rtf15Writer(document, target, fontFamily)
        return writer.go()


    def __init__(self, doc, target, family):
        self.document = doc
        self.target = target

        if family not in self.fonts:
            raise ValueError("Family %s not found (Try %s)" % (
                family, " or ".join("'%s'" % fam for fam in self.fonts)))

        self.fontFamily = family

        self._paragraphDispatch = {
            document.List: self._list,
            document.Paragraph: self._paragraph
        }


    def go(self):
        self.listLevel = -1
        self.addSpacing = None

        self.target.write('{')
        self._writeHeader()
        self._writeDocument()
        self.target.write('}')
        return self.target



    # -----------------------------------------------
    # Header section


    def _writeHeader(self):
        # Do this first to get the default font number
        fontTable = self._getFontTable()

        self.target.write(r'\rtf1\ansi\deff%s' % self.fontNumber);

        # Not strictly necessary
        self.target.write('\n')

        for part in (fontTable,
                     self._getColorTable(),
                     self._getStyleSheet(),
                     self._getListTable(),
                     self._getListOverrides(),
                     self._getRevTable()):

            if part:
                self.target.write(part)
                self.target.write('\n')



    def _getFontTable(self):
        output = [r'{\fonttbl']
        for i, (fontFamily, fontName) in enumerate(six.iteritems(self.fonts)):
            output.append(r'{\f%d\f%s %s;}' % (i, fontFamily, fontName))
            if fontFamily == self.fontFamily:
                self.fontNumber = i

        # We need Symbol for list bullets
        output.append(r'{\f%d\fnil\fprq0\fcharset128 Symbol;}' % (i+1))
        self.symbolFontNumber = i+1

        output.append('}')
        return "".join(output)


    def _getColorTable(self):
        # We only need black, and blue (for hyperlinks)
        return (r'{\colortbl;'
                r'\red0\green0\blue0;'
                r'\red0\green0\blue255;}')


    def _getStyleSheet(self):
        # OpenOffice won't render bullets unless there's a stylesheet entry
        # even if it doesn't do anything.
        return r'''{\stylesheet{\s1 List Paragraph;}}'''


    def _getListTable(self):
        # levelnfc23 means bullets (rather than numbering)
        # leveljc0 means left justified
        # levelfollow0 means a tab after the bullet
        output = [r'{\*\listtable{\list\listid1\listtemplateid1']

        for i in range(9):
            output.append((
                r'{\listlevel\levelstartat1\levelnfc23\leveljc0\levelfollow0'
                r'{\leveltext \'01\u61623 ?;}' # The bullet character
                r'\fi-180\f%d' # Indent the bullet left, and use the symbol font
                '}') % self.symbolFontNumber)

        output.append('}}')
        return "".join(output)


    def _getListOverrides(self):
        # I have no idea what the point is of this,
        # but we need it.
        return r'{\listoverridetable{\listoverride\listid1\listoverridecount0\ls0}}'


    def _getRevTable(self):
        # Hell no I don't think so
        pass


    # -----------------------------------------------
    # Document section


    def _writeDocument(self):

        for part in (self._getInfo(),
                     self._getDocFormat(),
                     self._getSecFormat()):

            if part:
                self.target.write(part)
                self.target.write('\n')


        for paragraph in self.document.content:
            handler = self._paragraphDispatch[paragraph.__class__]
            handler(paragraph)


    def _getInfo(self):
        pass


    def _getDocFormat(self):
        pass


    def _getSecFormat(self):
        pass



    # -----------------------------------------------
    # Content


    def _paragraph(self, paragraph, spacing=PARAGRAPH_SPACING):

        if self.addSpacing is not None:
            self.target.write(r'\sb%d' % self.addSpacing)
            self.addSpacing = None

        # Space after the paragraph,
        # expressed in units of god-knows-what
        self.target.write(r'\sa%d{' % spacing)

        for text in paragraph.content:
            self._text(text)

        self.target.write(r'}\par\pard' '\n')


    def _list(self, lst, spacing=PARAGRAPH_SPACING):
        self.listLevel += 1

        for entry in lst.content:
            for paragraph in entry.content:
                # It doesn't seem like RTF supports multiple paragraphs
                # in the same list item, so just let them be an item each.
                self.target.write(r'\ilvl%d\ls0\li%d\s1' % (
                    self.listLevel, 720*(self.listLevel+1)))
                handler = self._paragraphDispatch[paragraph.__class__]
                handler(paragraph, spacing=LIST_ITEM_SPACING)

        self.listLevel -= 1

        # When going back from a list to regular paragraphs,
        # add some extra spacing to balance the list out.
        if self.listLevel == -1:
            self.addSpacing = 150


    def _text(self, text):

        if 'url' in text.properties:
            self.target.write(
                r'{\field{\*\fldinst HYPERLINK %s}{\fldrslt \*\cf2\ul '
                % text.properties['url'])

        props = []

        if 'super' in text.properties:
            self.target.write('{\up9 ')
        elif 'sub' in text.properties:
            self.target.write('{\dn9 ')

        for prop in text.properties:
            if prop in _styleFlags:
                props.append(_styleFlags[prop])

        if props:
            self.target.write("".join(props) + " ")


        for run in text.content:
            for unichar in run:
                if unichar == '\n':
                    self.target.write(r'\line ')
                    continue

                point = ord(unichar)
                if point < 128:
                    self.target.write(str(unichar))
                else:
                    self.target.write(r'\u%d?' % point)

        if props:
            self.target.write("".join("%s0" % p for p in props) + " ")

        if 'super' in text.properties or 'sub' in text.properties:
            self.target.write("}")

        if 'url' in text.properties:
            self.target.write('}}')



import argparse
from pygov_br import __version__


def get_parser():
    """
    Creates a new argument parser.
    """

    parser = argparse.ArgumentParser('pygov-br')
    parser.add_argument('--version', '-v', action='version',
                        version='%(prog)s ' + __version__)
    return parser


def main(args=None):
    """Main entry point for your project.

    Parameters
    ----------

    args : list
        A of arguments as if they were input in the command line. Leave it None
        use sys.argv.
    """

    parser = get_parser()
    args = parser.parse_args(args)

    # Put your main script logic here
    print('List of arguments:')
    print(args)


if __name__ == '__main__':
    main(['-h'])




"""
Parse a css document into a python class that can be used to apply the
style to a BeautifulSoup document.
"""
from __future__ import absolute_import

import re


class Selector(object):
    """
    Represent a css selector.

    The __call__ method takes a BeautifulSoup node as argument end
    return True if the selector applies to the node.
    """

    def __init__(self, tag=None, klass=None):
        self.tag = tag
        self.klass = klass

    def check_tag(self, node):
        return not self.tag or node.findParent(self.tag)

    def check_class(self, node):
        return not self.klass or node.findParent(attrs={'class': self.klass})

    def __call__(self, node):
        return self.check_tag(node) and self.check_class(node)

    def __repr__(self):
        tag = self.tag if self.tag else ""
        klass = ".%s" % self.klass if self.klass else ""
        return "%s%s" % (tag, klass)


class Rule(object):
    """
    Represents a css rule.

    A rule consists of a selector and a dictionary of properties.
    """

    def __init__(self, selector, properties=None):
        self.selector = selector
        self.properties = properties or {}

    def __repr__(self):
        return "%s %s" % (self.selector, self.properties)


class CSS(object):
    """
    Represents a css document
    """

    # The regular expressions used to parse the css document
    # match a rule e.g: '.imp {font-weight: bold; color: blue}'
    ruleset_re = re.compile(r'\s*(.+?)\s+\{(.*?)\}')
    # match a property declaration, e.g: 'font-weight = bold'
    declaration_re = re.compile(r'\s*(.+?):\s*(.+?)\s*?(?:;|$)')
    # match a selector
    selector_re = re.compile(r'(.*?)(?:\.(.*))?$')

    def __init__(self, source=None):
        self.rules = []
        if source:
            self.parse_css(source)

    def __repr__(self):
        return repr(self.rules)

    def parse_css(self, css):
        """
        Parse a css style sheet into the CSS object.

        For the moment this will only work for very simple css
        documents.  It works by using regular expression matching css
        syntax.  This is not bullet proof.
        """
        rulesets = self.ruleset_re.findall(css)
        for (selector, declarations) in rulesets:
            rule = Rule(self.parse_selector(selector))
            rule.properties = self.parse_declarations(declarations)
            self.rules.append(rule)

    def parse_declarations(self, declarations):
        """
        parse a css declaration list
        """
        declarations = self.declaration_re.findall(declarations)
        return dict(declarations)

    def parse_selector(self, selector):
        """
        parse a css selector
        """
        tag, klass = self.selector_re.match(selector).groups()
        return Selector(tag, klass)

    def get_properties(self, node):
        """
        return a dict of all the properties of a given BeautifulSoup
        node found by applying the css style.
        """
        ret = {}
        # Try all the rules one by one
        for rule in self.rules:
            if rule.selector(node):
                ret.update(rule.properties)
        # Also search for direct 'style' arguments in the html doc
        for style_node in node.findParents(attrs={'style': True}):
            style = style_node.get('style')
            properties = self.parse_declarations(style)
            ret.update(properties)
        return ret

    def is_bold(self, node):
        """
        convenience method equivalent to
        self.get_properties(node).get('font-weight', None) == 'bold'
        """
        properties = self.get_properties(node)
        return properties.get('font-weight') == 'bold'

    def is_italic(self, node):
        properties = self.get_properties(node)
        return properties.get('font-style') == 'italic'

    def is_sub(self, node):
        properties = self.get_properties(node)
        return properties.get('vertical-align') == 'sub'

    def is_super(self, node):
        properties = self.get_properties(node)
        return properties.get('vertical-align') == 'super'




# -*- coding: utf-8 -*-
from pygov_br.base import Client
from datetime import datetime
from xml.etree.ElementTree import fromstring, ElementTree


class LegislativeBodyClient(Client):

    def __init__(self):
        host = 'http://www.camara.leg.br/SitCamaraWS/Orgaos.asmx/'
        super(LegislativeBodyClient, self).__init__(host)

    def all(self):
        """Fetch all legislative bodies.

        List all legislative bodies on Câmara dos Deputados.

        Returns:
            list: A list of all legislative bodies of Câmara dos Deputados
            (commissions, councils, etc). For example::

                [{'descricao': 'Comissão de Constituição e Justiça ' \
                               'e de Cidadania',
                  'id': 2003,
                  'idTipodeOrgao': 2,
                  'sigla': 'CCJC'}, ...]

        """
        xml_response = self._get('ObterOrgaos')
        list_response = self._xml_attributes_to_list(xml_response, 'orgao')
        return self._safe(list_response)

    def roles(self):
        """Fetch all legislative bodies roles.

        Returns:
            list: A list of all legislative bodies roles. For example::

            [{'descricao': 'Relator', 'id': 50}, ...]

        """
        xml_response = self._get('ListarCargosOrgaosLegislativosCD')
        list_response = self._xml_attributes_to_list(xml_response, 'cargo')
        return self._safe(list_response)

    def members(self, legislative_body_id):
        """Fetch members of a legislative body.

        Args:
            legislative_body_id (int): Legisltive body identifier.

        Returns:
            dict: A dictionary that contains 'membro', 'Presidente',
            'PrimeiroVice-Presidente', 'SegundoVice-Presidente' and
            'TerceiroVice-Presidente' as keys. The last three have a deputy
            dictionary as value and the first have a list of deputy dictionary.
            For example::

                    {'Presidente': {'ideCadastro': 73463,
                      'nome': 'Osmar Serraglio',
                      'partido': 'PMDB',
                      'situacao': 'Titular',
                      'uf': 'PR'},
                     'PrimeiroVice-Presidente': {'ideCadastro': 178897,
                      'nome': 'Rodrigo Pacheco',
                      'partido': 'PMDB',
                      'situacao': 'Titular',
                      'uf': 'MG'},
                     'SegundoVice-Presidente': {'ideCadastro': 93472,
                      'nome': 'Cristiane Brasil',
                      'partido': 'PTB',
                      'situacao': 'Titular',
                      'uf': 'RJ'},
                     'TerceiroVice-Presidente': {'ideCadastro': 178963,
                      'nome': 'Covatti Filho',
                      'partido': 'PP',
                      'situacao': 'Titular',
                      'uf': 'RS'},
                     'membro': [{'ideCadastro': 160559,
                       'nome': 'Alceu Moreira',
                       'partido': 'PMDB',
                       'situacao': 'Titular',
                       'uf': 'RS'}, ...]}

        """
        path = 'ObterMembrosOrgao?IDOrgao={0}'
        xml_response = self._get(path.format(legislative_body_id))
        element_tree = ElementTree(fromstring(xml_response.encode('utf-8')))
        members = element_tree.find('membros')
        dict_response = self._make_dict_from_tree(members)
        return self._safe(dict_response['membros'])

    def schedule(self, legislative_body_id, initial_date='', final_date=''):
        """Fetch all scheduled activities of a legislative body.

        Args:
            legislative_body_id (int): Legisltive body identifier.
            initial_date (str or datetime, optional): Initial date of period.
                If `str`, must be in the format: `dd/mm/yyyy`.
            final_date (str or datetime, optional): Final date of period.
                If `str`, must be in the format: `dd/mm/yyyy`.

        Returns:
            list: A list with all scheduled activities of a legislative body.
            For example::

                [{'codReuniao': 45639,
                  'comissao': 'CCJC - CONSTITUIÇÃO E JUSTIÇA E DE CIDADANIA',
                  'data': datetime.date(2016, 11, 10),
                  'estado': 'Convocada',
                  'horario': datetime.time(10, 0),
                  'local': 'Anexo II, Plenário 01',
                  'objeto': None,
                  'proposicoes': {'proposicao': [{
                    'ementa': 'Altera a Lei nº 9.472, de 16 de julho de 1997' \
                              ' permitindo à Anatel alterar a modalidade de ' \
                              'licenciamento de serviço de telecomunicações ' \
                              'de concessão para autorização.',
                    'idProposicao': 2025543,
                    'numOrdemApreciacao': 113,
                    'partidoRelator': 'PMDB',
                    'relator': 'Deputado Sergio Souza',
                    'resultado': None,
                    'sigla': 'PL 3453/2015',
                    'textoParecerRelator':
                        'Parecer do Relator, Dep. Sergio Souza (PMDB-PR), ' \
                        'pela constitucionalidade, juridicidade e técnica ' \
                        'legislativa deste, das Emendas da Comissão de ' \
                        'Ciência e Tecnologia, Comunicação e Informática e ' \
                        do Substitutivo da Comissão de Desenvolvimento ' \
                        Econômico, Indústria, Comércio e Serviços.',
                    'ufRelator': 'PR'}, ...]},
                  'tipo': 'Reunião Deliberativa',
                  'tituloReuniao': 'Reunião Deliberativa Ordinária'},

        """
        if isinstance(initial_date, datetime):
            initial_date = initial_date.strftime('%d/%m/%Y')
        if isinstance(final_date, datetime):
            final_date = final_date.strftime('%d/%m/%Y')

        path = "ObterPauta?IDOrgao={0}&datIni={1}&datFim={2}"
        xml_response = self._get(path.format(legislative_body_id,
                                             initial_date, final_date))
        dict_response = self._xml_to_dict(xml_response)
        list_response = dict_response['pauta']['reuniao']

        if isinstance(list_response, dict):
            list_response = [list_response]

        return self._safe(list_response)

    def types(self):
        """Fetch all legislative bodies types.

        Returns:
            list: A list of all legislative bodies types. For example::

            [{'descricao': 'Sociedade Civil', 'id': 70000}, ...]

        """
        xml_response = self._get('ListarTiposOrgaos')
        list_response = self._xml_attributes_to_list(xml_response, 'tipoOrgao')
        return self._safe(list_response)



# -*- coding: utf-8 -*-
from pygov_br.base import Client
from pygov_br.exceptions import MissingParameterError
from xml.etree.ElementTree import fromstring, ElementTree
from datetime import datetime


class ProposalClient(Client):

    def __init__(self):
        host = 'http://www.camara.gov.br/SitCamaraWS/Proposicoes.asmx/'
        super(ProposalClient, self).__init__(host)

    def filter(self, proposal_type='', year='', proposal_number='',
               initial_date='', final_date='', author_name='',
               author_type_id='', author_party_initials='', author_region='',
               author_gender='', status_id='', legislative_body_id='',
               in_progress=''):
        r"""Fetch filtered proposals.

        The `proposal_type` and `year` parameters are mandatory if
        `author_name` is empty.

        Args:
            proposal_type (str): Proposal type initials.
            year (int): Proposal's year.
            author_name (str): Part of author name.
            proposal_number (int, optional): Proposal number.
            initial_date (str or datetime, optional): Initial date of period.
                If `str`, must be in the format: `dd/mm/yyyy`.
            final_date (str or datetime, optional): Final date of period. If
                `str`,must be in the format: `dd/mm/yyyy`.
            author_type_id (int, optional): Author type identifier.
            author_party_initials (str, optional): Author's party initials
                identifier.
            author_gender (str, optional): Author gender. `M` for male or `F`
                for female.
            status_id (int, optional): Proposal status identifier.
            legislative_body_id (int, optional): Legislative body identifier.
            in_progress (int, optional): Proposal progress indication. `1` to
                `in progress` and `2` to `closed processing`.

        Returns:
            list: A list of the filtered proposals. Example::

                [{'ano': 2016,
                  'apreciacao': {'id': 5,
                   'txtApreciacao': 'Proposição Sujeita à Apreciação do ' \
                                    'Plenário'},
                  'autor1': {'codPartido': None,
                   'idecadastro': None,
                   'txtNomeAutor': 'Poder Executivo',
                   'txtSiglaPartido': None,
                   'txtSiglaUF': None},
                  'datApresentacao': datetime.datetime(2016, 11, 4, 18, 2),
                  'id': 2116055,
                  'indGenero': 'o',
                  'nome': 'PL 6427/2016',
                  'numero': 6427,
                  'orgaoNumerador': {'id': 180, 'nome': 'PLENÁRIO',
                                     'sigla': 'PLEN'},
                  'qtdAutores': 1,
                  'qtdOrgaosComEstado': 0,
                  'regime': {'codRegime': 22,
                             'txtRegime': 'Urgência art. 64 CF'},
                  'situacao': {'descricao': None,
                   'id': None,
                   'orgao': {'codOrgaoEstado': None, 'siglaOrgaoEstado': None},
                   'principal': {'codProposicaoPrincipal': 0,
                                 'proposicaoPrincipal': None}},
                  'tipoProposicao': {'id': 139, 'nome': 'Projeto de Lei',
                                     'sigla': 'PL'},
                  'txtEmenta': 'Altera a Lei nº 8.213, de 24 de julho de ' \
                               '1991, que dispõe sobre os Planos de ' \
                               'Benefícios da Previdência Social, e institui' \
                               'o Bônus Especial de Desempenho Institucional' \
                               ' por Perícia Médica em Benefícios por ' \
                               Incapacidade.',
                  'txtExplicacaoEmenta': None,
                  'ultimoDespacho': {'datDespacho': datetime.date(2016, 11, 4),
                   'txtDespacho': 'Às Comissões de Trabalho, de ' \
                                  'Administração e Serviço Público; ' \
                                  'Seguridade Social e Família; Finanças e ' \
                                  'Tributação (Art. 54 RICD) e Constituição ' \
                                  'e Justiça e de Cidadania (Art. 54 RICD)' \
                                  'Proposição Sujeita à Apreciação do ' \
                                  'Plenário. Regime de Tramitação: Urgência ' \
                                  'art. 64 CF'}}, ...]

        Raises:
            MissingParameterError: If `author_name` is empty AND
                `proposal_type` OR `year` OR both are empty too.
            ValueError: `author_gender` is not `M` or `F` or if `in_progress`
                is not `1` or `2`

        """
        proposal_type_or_year = proposal_type == '' or year == ''
        if author_name == '' and proposal_type_or_year:
            raise MissingParameterError(
                "The 'proposal_type' and 'year' parameters are mandatory if "
                "'author_name' is empty."
            )

        if author_gender not in ['M', 'm', 'F', 'f', '']:
            raise ValueError("'author_gender' must be 'M' or 'F'.")

        if in_progress not in [1, 2, '']:
            raise ValueError("'in_progress' must be '1' or '2'.")

        if isinstance(initial_date, datetime):
            initial_date = initial_date.strftime('%d/%m/%Y')
        if isinstance(final_date, datetime):
            final_date = final_date.strftime('%d/%m/%Y')

        path = "ListarProposicoes?sigla={0}&numero={1}&ano={2}&" \
               "datApresentacaoIni={3}&datApresentacaoFim={4}&" \
               "parteNomeAutor={5}&idTipoAutor={6}&siglaPartidoAutor={7}&" \
               "siglaUFAutor={8}&generoAutor={9}&codEstado={10}&" \
               "codOrgaoEstado={11}&emTramitacao={12}"
        xml_response = self._get(path.format(
            proposal_type, proposal_number, year, initial_date, final_date,
            author_name, author_type_id, author_party_initials, author_region,
            author_gender, status_id, legislative_body_id, in_progress
        ))
        dict_response = self._xml_to_dict(xml_response)
        list_response = dict_response['proposicoes']['proposicao']
        if isinstance(list_response, dict):
            list_response = [list_response]

        return self._safe(list_response)

    def get(self, proposal_type, proposal_number, year):
        r"""Fetch detailed proposal.

        Args:
            proposal_type (str): Proposal type initials.
            proposal_number (int): Proposal number.
            year (int): Proposal's year.

        Returns:
            dict: A detailed information about one proposal. Example::

                {'Apreciacao': 'Proposição Sujeita à Apreciação do Plenário',
                 'Autor': 'João Paulo Cunha',
                 'DataApresentacao': datetime.date(2011, 11, 16),
                 'Ementa': 'Altera a Lei nº 9.503, de 23 de setembro de 1997' \
                           'que institui o Código de Trânsito Brasileiro, ' \
                           'para dispor sobre a suspensão imediata do ' \
                           'direito de dirigir do condutor envolvido em ' \
                           'acidente de trânsito com vítima fatal, sob a ' \
                           'influência de álcool.',
                 'ExplicacaoEmenta': '',
                 'Indexacao': 'Alteração, Código de Trânsito Brasileiro, ' \
                              'suspensão, habilitação, condutor, veículo ' \
                              'automotor, ingestão, bebida alcoólica, ' \
                              'acidente de trânsito, morte, vítima.',
                 'LinkInteiroTeor': 'http://www.camara.gov.br/proposicoesWeb' \
                                    '/prop_mostrarintegra?codteor=939420',
                 'RegimeTramitacao': 'Urgência art. 155 RICD',
                 'Situacao': 'ARQUIVO - Arquivada',
                 'UltimoDespacho': 'Apense-se à(ao) PL-2473/2011.\n' \
                                   'Proposição Sujeita à Apreciação do ' \
                                   'Plenário\nRegime de Tramitação: ' \
                                   Prioridade',
                 'apensadas': None,
                 'idProposicao': 527478,
                 'idProposicaoPrincipal': '',
                 'ideCadastro': 73534,
                 'nomeProposicao': 'PL 2718/2011',
                 'nomeProposicaoOrigem': '',
                 'partidoAutor': 'PT',
                 'tema': 'Desenvolvimento Urbano e Trânsito',
                 'tipoProposicao': 'Projeto de Lei',
                 'ufAutor': 'SP'}

        """
        path = "ObterProposicao?tipo={0}&numero={1}&ano={2}"
        xml_response = self._get(path.format(proposal_type, proposal_number,
                                             year))
        dict_response = self._xml_to_dict(xml_response)
        return self._safe(dict_response['proposicao'])

    def get_by_id(self, proposal_id):
        r"""Fetch detailed proposal by id.

        Args:
            proposal_id (int): Proposal identifier.]

        Returns:
            dict: A detailed information about one proposal. Example::

                {'Apreciacao': 'Proposição Sujeita à Apreciação do Plenário',
                 'Autor': 'João Paulo Cunha',
                 'DataApresentacao': datetime.date(2011, 11, 16),
                 'Ementa': 'Altera a Lei nº 9.503, de 23 de setembro de 1997' \
                           'que institui o Código de Trânsito Brasileiro, ' \
                           'para dispor sobre a suspensão imediata do ' \
                           'direito de dirigir do condutor envolvido em ' \
                           'acidente de trânsito com vítima fatal, sob a ' \
                           'influência de álcool.',
                 'ExplicacaoEmenta': '',
                 'Indexacao': 'Alteração, Código de Trânsito Brasileiro, ' \
                              'suspensão, habilitação, condutor, veículo ' \
                              'automotor, ingestão, bebida alcoólica, ' \
                              'acidente de trânsito, morte, vítima.',
                 'LinkInteiroTeor': 'http://www.camara.gov.br/proposicoesWeb' \
                                    '/prop_mostrarintegra?codteor=939420',
                 'RegimeTramitacao': 'Urgência art. 155 RICD',
                 'Situacao': 'ARQUIVO - Arquivada',
                 'UltimoDespacho': 'Apense-se à(ao) PL-2473/2011.\n' \
                                   'Proposição Sujeita à Apreciação do ' \
                                   'Plenário\nRegime de Tramitação: ' \
                                   Prioridade',
                 'apensadas': None,
                 'idProposicao': 527478,
                 'idProposicaoPrincipal': '',
                 'ideCadastro': 73534,
                 'nomeProposicao': 'PL 2718/2011',
                 'nomeProposicaoOrigem': '',
                 'partidoAutor': 'PT',
                 'tema': 'Desenvolvimento Urbano e Trânsito',
                 'tipoProposicao': 'Projeto de Lei',
                 'ufAutor': 'SP'}

        """
        path = "ObterProposicaoPorID?IdProp={0}"
        xml_response = self._get(path.format(proposal_id))
        dict_response = self._xml_to_dict(xml_response)
        return self._safe(dict_response['proposicao'])

    def voting(self, proposal_type, proposal_number, year):
        r"""Fetch all votings of a proposal.

        Args:
            proposal_type (str): Proposal type initials.
            proposal_number (int): Proposal number.
            year (int): Proposal's year.

        Returns:
            list: A list of a specified proposal votings. Example:

                [{'Data': datetime.date(2012, 2, 29),
                  'Hora': datetime.time(19, 9),
                  'ObjVotacao': 'DVS - DEM - EMENDA 26',
                  'Resumo': 'Rejeitada a Emenda nº 26. Sim: 11; não: 275; ' \
                            'abstenção:02; total: 388.',
                  'codSessao': 4533,
                  'orientacaoBancada': [{'Sigla': 'PT', 'orientacao': 'Não'},
                   {'Sigla': 'PMDB', 'orientacao': 'Não'}, ...],
                  'votos': [
                   {'Nome': 'Nilson Leitão',
                    'Partido': 'PSDB',
                    'UF': 'MT',
                    'Voto': 'Sim',
                    'ideCadastro': 166401}, ...]
                 }, ...]

        """
        path = "ObterVotacaoProposicao?tipo={0}&numero={1}&ano={2}"
        xml_response = self._get(path.format(proposal_type, proposal_number,
                                             year))
        element_tree = ElementTree(fromstring(xml_response.encode('utf-8')))

        voting = element_tree.find('Votacoes')
        voting_list = []
        for child in voting.getchildren():
            voting_dict = child.attrib
            seat_orientation_list = []
            for orientation in child.find('orientacaoBancada').getchildren():
                seat_orientation_list.append(orientation.attrib)
            voting_dict['orientacaoBancada'] = seat_orientation_list

            votes_list = []
            for vote in child.find('votos').getchildren():
                votes_list.append(vote.attrib)
            voting_dict['votos'] = votes_list

            voting_list.append(voting_dict)

        return self._safe(voting_list)

    def voted(self, year, proposal_type=''):
        """Fetch all voted proposals in the year.

        Args:
            year (int): Proposal's year.
            proposal_type (str): Proposal type initials.

        Returns:
            list: a list of proposals that was voted in the year. Example::

                [{'codProposicao': 2078295,
                  'dataVotacao': datetime.date(2016, 10, 5),
                  'nomeProposicao': 'PL 4567/2016'}, ...]

        """
        path = 'ListarProposicoesVotadasEmPlenario?ano={0}&tipo={1}'
        xml_response = self._get(path.format(year, proposal_type))
        dict_response = self._xml_to_dict(xml_response)
        list_response = dict_response['proposicoes']['proposicao']

        if isinstance(list_response, dict):
            list_response = [list_response]
        return self._safe(list_response)

    def processed_in_period(self, initial_date, final_date):
        """Fetch all processed proposals in the period.

        Args:
            initial_date (str or datetime): Initial date of period. If `str`,
                must be in the format: `dd/mm/yyyy`.
            final_date (str or datetime): Final date of period. If `str`,
                must be in the format: `dd/mm/yyyy`.

        Returns:
            list: A list of proposal that was processed in the period.
            Example::

                [{'ano': 2013,
                  'codProposicao': 590279,
                  'dataAlteracao': datetime.datetime(2013, 10, 1, 11, 16, 28),
                  'dataTramitacao': datetime.date(2013, 9, 20),
                  'numero': 47,
                  'tipoProposicao': 'SIT'}, ...]

        """
        if isinstance(initial_date, datetime):
            initial_date = initial_date.strftime('%d/%m/%Y')
        if isinstance(final_date, datetime):
            final_date = final_date.strftime('%d/%m/%Y')

        path = 'ListarProposicoesTramitadasNoPeriodo?dtInicio={0}&dtFim={1}'
        xml_response = self._get(path.format(initial_date, final_date))
        dict_response = self._xml_to_dict(xml_response)
        list_response = dict_response['proposicoes']['proposicao']

        if isinstance(list_response, dict):
            list_response = [list_response]
        return self._safe(list_response)

    def progress(self, proposal_number, year, proposal_type='',
                 initial_date='', legislative_body_id=''):
        r"""Fetch the progress of a proposal.

        Args:
            proposal_number (int): Proposal number.
            year (int): Proposal's year.
            proposal_type (str, optional): Proposal type initials.
            initial_date (str or datetime, optional): Initial date of period.
                If `str`, must be in the format: `dd/mm/yyyy`.
            legislative_body_id (int, optional): Legislative body identifier.

        Returns:
            dict: A dictc with progress steps of a proposal.

                {'andamento': {'tramitacao': [{'codOrgao': 4,
                    'data': datetime.date(2009, 12, 23),
                    'descricao': 'Transformado na Lei Ordinária 12154/2009.' \
                                 'DOU 24 12 09 PÁG 01 COL 01. Vetado ' \
                                 'parcialmente. Razões do veto: MSC ' \
                                 '1.085/09-PE. DOU (Edição extra) 24 12 09' \
                                 'PÁG 8 COL 02.',
                    'inteiroTeor': None,
                    'ordemDeTramitacao': 87,
                    'orgao': 'MESA'}, ...]},
                 'ementa': 'Cria a Superintendência Nacional de Previdência' \
                           'Complementar - PREVIC e dispõe sobre o seu ' \
                           'pessoal, inclui a Câmara de Recursos da ' \
                           'Previdência Complementar na estrutura básica do' \
                           'Ministério da Previdência Social, altera ' \
                           'disposições referentes a auditores-fiscais da' \
                           'Receita Federal do Brasil, e dá outras' \
                           'providências.',
                 'idProposicao': 408406,
                 'situacao': 'Tranformada no(a) Lei Ordinária 12154/2009',
                 'ultimaAcao': {'tramitacao': {'codOrgao': 4,
                   'data': datetime.date(2010, 2, 10),
                   'descricao': 'Recebimento do Ofício nº 60/10 (CN) ' \
                                'comunicando veto parcial e solicitando ' \
                                'indicação de membros para integrar a ' \
                                'Comissão Mista incumbida de relatar o(s) ' \
                                'veto(s).',
                   'ordemDeTramitacao': 89,
                   'orgao': 'MESA - Mesa Diretora da Câmara dos Deputados'}}}

        """
        if isinstance(initial_date, datetime):
            initial_date = initial_date.strftime('%d/%m/%Y')

        path = 'ObterAndamento?sigla={0}&numero={1}&ano={2}&' \
               'dataIni={3}&codOrgao={4}'
        xml_response = self._get(
            path.format(proposal_type, proposal_number, year,
                        initial_date, legislative_body_id),
            host='http://www.camara.leg.br/SitCamaraWS/Orgaos.asmx/')
        dict_response = self._xml_to_dict(xml_response)
        return self._safe(dict_response['proposicao'])

    def amendments(self, proposal_type, proposal_number, year):
        """Fetch amendments of a proposal.

        Args:
            proposal_type (str): Proposal type initials.
            proposal_number (int): Proposal number.
            year (int): Proposal's year.

        Returns:
            list: A list of amendments of a specific proposal. Example::

                [{'CodProposicao': 413970,
                  'Descricao': 'EMC 2/2008 CSSF => PL 3962/2008'}, ...]

        """
        path = 'ObterEmendasSubstitutivoRedacaoFinal?' \
               'tipo={0}&numero={1}&ano={2}'
        xml_response = self._get(
            path.format(proposal_type, proposal_number, year),
            host='http://www.camara.leg.br/SitCamaraWS/Orgaos.asmx/')
        element_tree = ElementTree(fromstring(xml_response.encode('utf-8')))

        return self._safe(self._tree_attributes_to_list(element_tree,
                                                        'Emendas'))

    def final_wordings(self, proposal_type, proposal_number, year):
        """Fetch final wordings of a proposal.

        Args:
            proposal_type (str): Proposal type initials.
            proposal_number (int): Proposal number.
            year (int): Proposal's year.

        Returns:
            list: A list of final wordings of a specific proposal. Example::

                [{'CodProposicao': 440524,
                  'Descricao': 'RDF 1 => PL 3962/2008''}, ...]

        """
        path = 'ObterEmendasSubstitutivoRedacaoFinal?' \
               'tipo={0}&numero={1}&ano={2}'
        xml_response = self._get(
            path.format(proposal_type, proposal_number, year),
            host='http://www.camara.leg.br/SitCamaraWS/Orgaos.asmx/')
        element_tree = ElementTree(fromstring(xml_response))

        return self._safe(self._tree_attributes_to_list(element_tree,
                                                        'RedacoesFinais'))

    def substitutives(self, proposal_type, proposal_number, year):
        """Fetch substitutives of a proposal.

        Args:
            proposal_type (str): Proposal type initials.
            proposal_number (int): Proposal number.
            year (int): Proposal's year.

        Returns:
            list: A list of substitutives of a specific proposal. Example::

                [{'CodProposicao': 439782,
                  'Descricao': 'SBT 1 CTASP => PL 3962/2008'}]

        """
        path = 'ObterEmendasSubstitutivoRedacaoFinal?' \
               'tipo={0}&numero={1}&ano={2}'
        xml_response = self._get(
            path.format(proposal_type, proposal_number, year),
            host='http://www.camara.leg.br/SitCamaraWS/Orgaos.asmx/')
        element_tree = ElementTree(fromstring(xml_response))

        return self._safe(self._tree_attributes_to_list(element_tree,
                                                        'Substitutivos'))

    def comissions_opinion(self, proposal_type, proposal_number, year):
        r"""Fetch comissions opinion about a proposal.

        Args:
            proposal_type (str): Proposal type initials.
            proposal_number (int): Proposal number.
            year (int): Proposal's year.

        Returns:
            list: A list of comissions that will examine or was examine
        the proposal. Example::

                [{'dataParecer': datetime.datetime(2009, 6, 24, 10, 0),
                  'inteiroTeorParecerComissao': None,
                  'inteiroTeorParecerRelator':
                    'http://www.camara.gov.br/proposicoesWeb/' \
                    'prop_mostrarintegra?codteor=672715',
                  'parecer': 'Parecer proferido em Plenário pelo Relator, ' \
                             'Dep. Mendes Ribeiro Filho (PMDB-RS), pela ' \
                             'Comissão de Constituição e Justiça e de ' \
                             'Cidadania, que conclui pela ' \
                             'constitucionalidade, juridicidade e técnica ' \
                             legislativa das Emendas de Plenário de nºs ' \
                             1 e 2.',
                  'relator': 'Mendes Ribeiro Filho',
                  'tipoAnalise': 'Constitucionalidade'}, ...]
        """
        path = 'ObterIntegraComissoesRelator?tipo={0}&numero={1}&ano={2}'
        xml_response = self._get(
            path.format(proposal_type, proposal_number, year),
            host='http://www.camara.leg.br/SitCamaraWS/Orgaos.asmx/')
        element_tree = ElementTree(fromstring(xml_response.encode('utf-8')))
        dict_response = self._make_dict_from_tree(
            element_tree.find('comissoes')
        )
        list_response = dict_response['comissoes']['comissao']
        if isinstance(list_response, dict):
            list_response = [list_response]

        return self._safe(list_response)

    def types(self):
        """Fetch all proposal types.

        Returns:
            list: A list of proposal types. Example::

                [{'ativa': True,
                  'descricao': 'Voto em Separado',
                  'genero': 'o',
                  'tipoSigla': 'VTS'}, ...]
        """
        xml_response = self._get('ListarSiglasTipoProposicao')
        dict_response = self._xml_attributes_to_list(xml_response, 'sigla')
        return self._safe(dict_response)

    def author_types(self):
        """Fetch all author types.

        Returns:
            list: A list of author types. Example::

                [{'descricao': 'Deputado', 'id': 'TipoParlamentar_10000'}, ...]
        """
        xml_response = self._get('ListarTiposAutores')
        dict_response = self._xml_attributes_to_list(xml_response, 'TipoAutor')
        return self._safe(dict_response)

    def statuses(self):
        """Fetch all proposal statuses.

        Returns:
            list: A list of proposal statuses. Example::

                [{'ativa': True, 'descricao': 'Arquivada', 'id': 923}, ...]
        """
        xml_response = self._get('ListarSituacoesProposicao')
        dict_response = self._xml_attributes_to_list(xml_response,
                                                     'situacaoProposicao')
        return self._safe(dict_response)



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-07-19 23:07
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('camara_deputados', '0002_auto_20170719_2250'),
    ]

    operations = [
        migrations.AlterField(
            model_name='proposal',
            name='summary',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='proposal',
            name='summary_explanation',
            field=models.TextField(blank=True, null=True),
        ),
    ]



"""
Render documents as plaintext.
"""
from __future__ import absolute_import

from pygov_br.vendor.pyth import document
from pygov_br.vendor.pyth.format import PythWriter

from io import StringIO

class PlaintextWriter(PythWriter):

    @classmethod
    def write(klass, document, target=None, encoding="utf-8", newline="\n"):
        if target is None:
            target = StringIO()

        writer = PlaintextWriter(document, target, encoding, newline.encode(encoding))
        return writer.go()


    def __init__(self, doc, target, encoding, newline):
        self.document = doc
        self.target = target
        self.encoding = encoding
        self.newline = newline
        self.indent = -1
        self.paragraphDispatch = {
            document.List: self.list,
            document.Paragraph: self.paragraph
        }


    def go(self):
        for (i, paragraph) in enumerate(self.document.content):
            handler = self.paragraphDispatch[paragraph.__class__]
            handler(paragraph)
            self.target.write(self.newline)

        # Heh heh, remove final paragraph spacing
        self.target.seek(-2, 1)
        self.target.truncate()

        self.target.seek(0)
        return self.target


    def paragraph(self, paragraph, prefix=""):
        content = []
        for text in paragraph.content:
            content.append(u"".join(text.content))
        content = u"".join(content).encode(self.encoding)

        for line in content.splitlines():
            self.target.write("  ".encode(self.encoding) * self.indent)
            self.target.write(prefix.encode(self.encoding))
            self.target.write(line)
            self.target.write(self.newline)
            if prefix: prefix = "  "


    def list(self, list, prefix=None):
        self.indent += 1
        for (i, entry) in enumerate(list.content):
            for (j, paragraph) in enumerate(entry.content):
                prefix = "* " if j == 0 else "  "
                handler = self.paragraphDispatch[paragraph.__class__]
                handler(paragraph, prefix)
        self.indent -= 1








"""
Stuff for format implementations to subclass / use.
"""


class PythReader(object):
    """
    Base class for all Pyth readers.

    Readers must implement these methods.
    """

    @classmethod
    def read(self, source):
        """
        source: An object to read the document from.
        Usually (but not necessarily) a file object.

        Returns: A pyth.document.Document object.
        """
        pass



class PythWriter(object):
    """
    Base class for all Pyth writers.

    Writers must implement these methods.
    """

    @classmethod
    def write(self, document, target=None):
        """
        document: An instance of pyth.document.Document
        
        target: An object to write the document to.
        Usually (but not necessarily) a file object.
        If target is None, return something sensible
        (like a StringIO object)

        Returns: The target object
        """
        pass



"""
Abstract document representation
"""
from __future__ import absolute_import
import six

class _PythBase(object):

    def __init__(self, properties={}, content=[]):
        self.properties = {}
        self.content = []
        
        for (k,v) in six.iteritems(properties):
            self[k] = v

        for item in content:
            self.append(item)


    def __setitem__(self, key, value):
        if key not in self.validProperties:
            raise ValueError("Invalid %s property: %s" % (self.__class__.__name__, repr(key)))

        self.properties[key] = value

    def __getitem__(self, key):
        if key not in self.validProperties:
            raise ValueError("Invalid %s property: %s" %
                             (self.__class__.__name__, repr(key)))
        return self.properties.get(key)

    def append(self, item):
        """
        Try to add an item to this element.

        If the item is of the wrong type, and if this element has a sub-type,
        then try to create such a sub-type and insert the item into that, instead.
        
        This happens recursively, so (in python-markup):
          L [ u'Foo' ]
        actually creates:
          L [ LE [ P [ T [ u'Foo' ] ] ] ]

        If that doesn't work, raise a TypeError.
        """

        okay = True
        if not isinstance(item, self.contentType):
            if hasattr(self.contentType, 'contentType'):
                try:
                    item = self.contentType(content=[item])
                except TypeError:
                    okay = False
            else:
                okay = False
                
        if not okay:
            raise TypeError("Wrong content type for %s: %s (%s)" % (
                self.__class__.__name__, repr(type(item)), repr(item)))

        self.content.append(item)


class Text(_PythBase):
    """
    Text runs are strings of text with markup properties,
    like 'bold' or 'italic' (or 'hyperlink to ...').

    They are rendered inline (not as blocks).

    They do not inherit their properties from anything.
    """

    validProperties = ('bold', 'italic', 'underline', 'url', 'sub', 'super', 'strike')
    contentType = six.text_type

    def __repr__(self):
        return "Text('%s' %s)" % ("".join("[%s]" % r.encode("utf-8") for r in self.content), self.properties)


class Paragraph(_PythBase):
    """
    Paragraphs contain zero or more text runs.

    They cannot contain other paragraphs (but see List).

    They have no text markup properties, but may
    have rendering properties (e.g. margins)
    """

    validProperties = ()
    contentType = Text


class Image(Paragraph):
    """
    An image is stored in bytes. All properties of images from the rtf definition are allowed.
    """
    
    validProperties = ('emfblip', 'pngblip', 'jpegblip', 'macpict', 'pmmetafile', 'wmetafile', 'dibitmap', 
                       'wbitmap', 'wbmbitspixel', 'wbmplanes', 'wbmwidthbytes', 'picw', 'pich', 'picwgoal', 
                       'pichgoal', 'picscalex', 'picscaley', 'picscaled', 'piccropt', 'piccropb', 'piccropr', 
                       'piccropl', 'picbmp', 'picbpp', 'bin', 'blipupi', 'blipuid', 'bliptag', 'wbitmap')
    contentType = bytes

    def __repr__(self):
        return "Image(%d bytes, %s)" %(len(self.content[0])/2,self.properties)


class ListEntry(_PythBase):
    """
    A list of paragraphs representing one item in a list
    """
    validProperties = ()
    contentType = Paragraph


class List(Paragraph):
    """
    A list of paragraphs which will be rendered as a bullet list.

    A List is a Paragraph, so Lists can be nested.
    """

    validProperties = ()
    contentType = ListEntry
    


class Document(_PythBase):
    """
    Top-level item. One document is exactly one file.
    Documents consist of a list of paragraphs.
    """
    
    validProperties = ('title', 'subject', 'author')
    contentType = Paragraph



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-07-19 22:50
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('camara_deputados', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Proposal',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('year', models.IntegerField()),
                ('number', models.IntegerField()),
                ('submission_date', models.DateField()),
                ('name', models.CharField(max_length=500)),
                ('summary', models.TextField()),
                ('summary_explanation', models.TextField()),
                ('author', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='camara_deputados.Deputy')),
            ],
            options={
                'verbose_name': 'Proposal',
                'verbose_name_plural': 'Proposals',
            },
        ),
        migrations.CreateModel(
            name='ProposalType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_active', models.BooleanField()),
                ('description', models.CharField(max_length=500)),
                ('gender', models.CharField(max_length=2)),
                ('initials', models.CharField(max_length=50)),
            ],
            options={
                'verbose_name': 'Proposal Type',
                'verbose_name_plural': 'Proposal Types',
            },
        ),
        migrations.AddField(
            model_name='proposal',
            name='proposal_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='camara_deputados.ProposalType'),
        ),
    ]



"""
Write Pyth documents straight in Python, a la Nevow's Stan.
"""
from __future__ import absolute_import

from pygov_br.vendor.pyth.format import PythReader
from pygov_br.vendor.pyth.document import *
import six


def _convert(content):
    if isinstance(content, _PythonBase):
        return content.toPyth()
    return content

class PythonReader(PythReader):

    @classmethod
    def read(self, source):
        """
        source: A list of P objects.
        """
        return Document(content=[_convert(c) for c in source])



class _Shortcut(object):
    def __init__(self, key):
        self.key = key

    def asDict(self):
        return dict(((self.key, True),))


BOLD = _Shortcut("bold")
ITALIC = _Shortcut("italic")
UNDERLINE = _Shortcut("underline")
SUPER = _Shortcut("super")
SUB = _Shortcut("sub")


def _MetaPythonBase():
    """
    Return a metaclass which implements __getitem__,
    allowing e.g. P[...] instead of P()[...]
    """

    class MagicGetItem(type):
        def __new__(mcs, name, bases, dict):
            klass = type.__new__(mcs, name, bases, dict)
            mcs.__getitem__ = lambda _, k: klass()[k]
            return klass

    return MagicGetItem



class _PythonBase(object):
    """
    Base class for Python markup objects, providing
    stan-ish interface
    """

    def __init__(self, *shortcuts, **properties):
        self.properties = properties.copy()

        for shortcut in shortcuts:
            self.properties.update(shortcut.asDict())

        self.content = []


    def toPyth(self):
        return self.pythType(self.properties,
                             [_convert(c) for c in self.content])


    def __getitem__(self, item):

        if isinstance(item, (tuple, list)):
            for i in item: self [i]
        elif isinstance(item, int):
            return self.content[item]
        else:
            self.content.append(item)

        return self


    def __str__(self):
        return "%s(%s) [ %s ]" % (
            self.__class__.__name__,
            ", ".join("%s=%s" % (k, repr(v)) for (k,v) in six.iteritems(self.properties)),
            ", ".join(repr(x) for x in self.content))



class P(six.with_metaclass(_MetaPythonBase(), _PythonBase)):
    pythType = Paragraph


class LE(six.with_metaclass(_MetaPythonBase(), _PythonBase)):
    pythType = ListEntry

class L(six.with_metaclass(_MetaPythonBase(), _PythonBase)):
    pythType = List


class T(six.with_metaclass(_MetaPythonBase(), _PythonBase)):
    __repr__ = _PythonBase.__str__
    pythType = Text

    def toPyth(self):
        return Text(self.properties, self.content)



# -*- coding: utf-8 -*-
from pygov_br.base import ClientWrapper, Client
from pygov_br.exceptions import ClientError, ClientServerError
from pygov_br.camara_deputados.deputy import DeputyClient
from xml.etree.ElementTree import fromstring, ElementTree
import datetime
import pytest
import responses


def test_client_wrapper_with_class():
    wrapper = ClientWrapper(deputy_client=DeputyClient)
    assert isinstance(wrapper.deputy_client, DeputyClient)


def test_client_wrapper_with_instance():
    wrapper = ClientWrapper(deputy_client=DeputyClient())
    assert isinstance(wrapper.deputy_client, DeputyClient)


def test_client_instance():
    client = Client('host', timeout=10)
    assert client.host == 'host'
    assert client.timeout == 10


@responses.activate
def test_client_get():
    responses.add(responses.GET, 'http://mock.com/path',
                  body='data', status=200)
    response = Client('http://mock.com/')._get('path')

    assert len(responses.calls) == 1
    assert response == 'data'


@responses.activate
def test_client_get_with_host():
    responses.add(responses.GET, 'http://mocktest.com/path',
                  body='data', status=200)
    response = Client('http://mock.com/')._get('path',
                                               host='http://mocktest.com/')

    assert len(responses.calls) == 1
    assert response == 'data'


@responses.activate
def test_client_get_error_404():
    responses.add(responses.GET, 'http://mock.com/path',
                  body='not found', status=404)
    with pytest.raises(ClientError, message='[404]: None'):
        Client('http://mock.com/')._get('path')

    assert len(responses.calls) == 1


@responses.activate
def test_client_get_error_500():
    responses.add(responses.GET, 'http://mock.com/path',
                  body='service unavailable', status=503)
    with pytest.raises(ClientServerError):
        Client('http://mock.com/')._get('path')

    assert len(responses.calls) == 1


def test_xml_attributes_to_list():
    xml_string = """
    <parent>
        <child id="1" description="test 1"/>
        <child id="2" description="test 2"/>
        <child id="3" description="test 3"/>
        <child id="4" description="test 4"/>
    </parent>
    """
    expected_list = [
        {'id': '1', 'description': 'test 1'},
        {'id': '2', 'description': 'test 2'},
        {'id': '3', 'description': 'test 3'},
        {'id': '4', 'description': 'test 4'},
    ]
    client = Client('http://mock.com/')
    result_list = client._xml_attributes_to_list(xml_string, 'child')
    assert result_list == expected_list


def test_tree_attributes_to_list():
    xml_string = """
    <grandparent>
        <parent>
            <child id="1" description="test 1"/>
            <child id="2" description="test 2"/>
            <child id="3" description="test 3"/>
            <child id="4" description="test 4"/>
        </parent>
    </grandparent>
    """
    expected_list = [
        {'id': '1', 'description': 'test 1'},
        {'id': '2', 'description': 'test 2'},
        {'id': '3', 'description': 'test 3'},
        {'id': '4', 'description': 'test 4'},
    ]
    element_tree = ElementTree(fromstring(xml_string))
    client = Client('http://mock.com/')
    result_list = client._tree_attributes_to_list(element_tree, 'parent')
    assert result_list == expected_list


def test_make_dict_from_tree():
    xml_string = """
    <parent>
        <child>Test1</child>
        <child>Test2</child>
        <child>Test3</child>
    </parent>
    """
    expected_dict = {
        'parent': {
            'child': ['Test1', 'Test2', 'Test3']
        }
    }
    client = Client('http://mock.com/')
    etree = ElementTree(fromstring(xml_string))
    result_dict = client._make_dict_from_tree(etree.getroot())
    assert result_dict == expected_dict


def test_make_dict_from_empty_tree():
    etree = ElementTree()
    client = Client('http://mock.com/')
    expected_dict = {}
    assert client._make_dict_from_tree(etree.getroot()) == expected_dict


def test_safe_str_element():
    client = Client('http://mock.com/')
    assert type(client._safe_element('String')) == str


def test_safe_int_element():
    client = Client('http://mock.com/')
    assert type(client._safe_element('1')) == int


def test_safe_float_element():
    client = Client('http://mock.com/')
    assert type(client._safe_element('1.2')) == float


def test_safe_bool_element():
    client = Client('http://mock.com/')
    assert type(client._safe_element('True')) == bool
    assert type(client._safe_element('False')) == bool
    assert type(client._safe_element('true')) == bool
    assert type(client._safe_element('false')) == bool


def test_safe_list_element():
    client = Client('http://mock.com/')
    assert type(client._safe_element([1, 2])) == list


def test_safe_dict_element():
    client = Client('http://mock.com/')
    assert type(client._safe_element({'key1': 'value'})) == dict


def test_safe_datetime_element():
    client = Client('http://mock.com/')
    assert isinstance(client._safe_element('10:30'), datetime.time)
    assert isinstance(client._safe_element('10:30:40'), datetime.time)
    assert isinstance(client._safe_element('10/10/2000'), datetime.date)
    assert isinstance(client._safe_element('10/10/2000 10:00'),
                      datetime.datetime)
    assert isinstance(client._safe_element('10/10/2000 10:00:12'),
                      datetime.datetime)


def test_safe_dict():
    client = Client('http://mock.com/')
    test_dict = {
        'int': '1',
        'float': '1.2',
        'str': 'string',
        'bool': 'True',
        'list': ['1', '2'],
        'dict': {'key': 'value'},
        'date': '10/10/2010',
    }
    expected_dict = {
        'int': 1,
        'float': 1.2,
        'str': 'string',
        'bool': True,
        'list': [1, 2],
        'dict': {'key': 'value'},
        'date': datetime.date(2010, 10, 10),
    }
    assert client._safe(test_dict) == expected_dict


def test_safe_list():
    client = Client('http://mock.com/')
    test_list = ['1', '1.2', 'string', 'True', ['1', '2'],
                 {'key': 'value'}, '10/10/2010']
    expected_list = [1, 1.2, 'string', True, [1, 2], {'key': 'value'},
                     datetime.date(2010, 10, 10)]
    assert client._safe(test_list) == expected_list


def test_safe_element():
    client = Client('http://mock.com/')
    assert type(client._safe('1')) == int
    assert type(client._safe('1.2')) == float
    assert type(client._safe('string')) == str
    assert type(client._safe('10:20')) == datetime.time
    assert type(client._safe('10:20:40')) == datetime.time
    assert type(client._safe('10/10/2010')) == datetime.date
    assert type(client._safe('10/10/2010 10:10')) == datetime.datetime
    assert type(client._safe('10/10/2010 10:10:20')) == datetime.datetime



"""
Read documents from RTF 1.5

http://www.biblioscape.com/rtf15_spec.htm

This module is potentially compatible with RTF versions up to 1.9.1,
but may not ignore all necessary control groups.

RTF is 8-bit based. Therefore, all input data is represented as
a binary string (with b prefix for literals).
Output data is unicode.
Python-related constants, pyth-internal constants and error messages are plain strings
(i.e. byte strings in Python 2, Unicode strings in Python 3):
"""
from __future__ import absolute_import
import string, re, itertools, struct

from pygov_br.vendor.pyth import document
from pygov_br.vendor.pyth.format import PythReader
from pygov_br.vendor.pyth.encodings import symbol
import six

_CONTROLCHARS = b'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-*'
_DIGITS = b'0123456789-'


_CODEPAGES = {
    0: "cp1252",   # ANSI
    1: "cp1252",   # Default (this is wrong, but there is no right)

    2: "symbol",   # Symbol
    77: "mac-roman", # Mac Roman

    # Does Python have built-in support for these? What is it?
    # 78: "10001", # Mac Shift Jis
    # 79: "10003", # Mac Hangul
    # 80: "10008", # Mac GB2312
    # 81: "10002", # Mac Big5
    # 83: "10005", # Mac Hebrew

    84: "mac-arabic", # Mac Arabic
    85: "mac-greek", # Mac Greek
    86: "mac-turkish", # Mac Turkish

    # 87: "10021", # Mac Thai
    # 88: "10029", # Mac East Europe
    # 89: "10007", # Mac Russian

    128: "cp932",  # Shift JIS
    129: "cp949",  # Hangul
    130: "cp1361", # Johab
    134: "cp936",  # GB2312
    136: "cp950",  # Big5
    161: "cp1253", # Greek
    162: "cp1254", # Turkish
    163: "cp1258", # Vietnamese
    177: "cp1255", # Hebrew
    178: "cp1256", # Arabic
    186: "cp1257", # Baltic
    204: "cp1251", # Russian
    222: "cp874",  # Thai
    238: "cp1250", # Eastern European
    254: "cp437",  # PC 437
    255: "cp850",  # OEM
}

# All the ones named by number in my 2.6 encodings dir, and those listed above
_CODEPAGES_BY_NUMBER = dict(
    (x, "cp%s" % x) for x in (37, 424, 437, 500, 737, 775, 850, 852, 855, 856,
                              857, 860, 861, 862, 863, 864, 865, 866, 869, 874,
                              875, 932, 936, 949, 950, 1006, 1026, 1140, 1250,
                              1251, 1252, 1253, 1254, 1255, 1256, 1257, 1258, 1361))

# Miscellaneous, incomplete
_CODEPAGES_BY_NUMBER.update({
    10000: "mac-roman",
    10007: "mac-greek"
})


class BackslashEscape(Exception):
    pass


class Rtf15Reader(PythReader):

    @classmethod
    def read(self, source, errors='strict', clean_paragraphs=True):
        """
        source: A list of P objects.
        """

        reader = Rtf15Reader(source, errors, clean_paragraphs)
        return reader.go()


    def __init__(self, source, errors='strict', clean_paragraphs=True):
        self.source = source
        self.errors = errors
        self.clean_paragraphs = clean_paragraphs
        self.document = document.Document


    def go(self):
        self.source.seek(0)

        if self.source.read(5) != br"{\rtf":
            from pygov_br.vendor.pyth.errors import WrongFileType
            raise WrongFileType("Doesn't look like an RTF file")

        self.source.seek(0)

        self.charsetTable = None
        self.charset = 'cp1252'
        self.group = Group(self)
        self.stack = [self.group]
        self.parse()
        return self.build()


    def parse(self):
        while True:
            nextbyte = self.source.read(1)

            if not nextbyte:
                break

            if nextbyte in b'\r\n':
                continue
            if nextbyte == b'{':
                subGroup = Group(self, self.group, self.charsetTable)
                self.stack.append(subGroup)
                subGroup.skip = self.group.skip
                self.group.flushChars()
                self.group = subGroup
            elif nextbyte == b'}':
                subGroup = self.stack.pop()
                self.group = self.stack[-1]
                if self.group.specialMeaning == 'FONT_TABLE':
                    subGroup.ignore()  # else font names may appear as text
                subGroup.finalize()

                if subGroup.specialMeaning == 'FONT_TABLE':
                    self.charsetTable = subGroup.charsetTable
                self.group.content.append(subGroup)

            elif self.group.skip:
                # Avoid crashing on stuff we can't handle
                # inside groups we don't care about anyway
                continue

            elif nextbyte == b'\\':
                control, digits = self.getControl()
                self.group.handle(control, digits)
            else:
                self.group.char(nextbyte)  # within-group text


    def getControl(self):
        chars = []  # for output 1
        digits = [] # for output 2
        current = chars  # what part we are reading
        is_first = True  # true during first char only
        while True:
            nextbyte = self.source.read(1)  # str in Python 2, bytes in Python 3

            if not nextbyte:
                break

            if is_first and nextbyte in b'\\{}':
                chars.append(b"control_symbol")
                digits.append(nextbyte)
                break

            if is_first and nextbyte in b'\r\n':
                # Special-cased in RTF, equivalent to a \par
                chars.append(b"par")
                break

            is_first = False

            if nextbyte == b"'":
                # ANSI escape, takes two hex digits
                chars.append(b"ansi_escape")
                digits.append(self.source.read(2))
                break

            if nextbyte == b' ':
                # Don't rewind, the space is just a delimiter
                break

            if nextbyte not in _CONTROLCHARS:
                # Rewind, it's a meaningful character
                self.source.seek(-1, 1)
                break

            if nextbyte in _DIGITS:
                current = digits

            current.append(nextbyte)

        return b"".join(chars), b"".join(digits)


    def build(self):
        doc = document.Document()

        ctx = DocBuilder(doc, self.clean_paragraphs)

        for bit in self.group.flatten():
            typeName = type(bit).__name__
            getattr(ctx, "handle_%s" % typeName)(bit)

        ctx.flushParagraph()

        return doc



class DocBuilder(object):

    def __init__(self, doc, clean_paragraphs=True):
        self.run = []
        self.propStack = [{}]
        self.block = None

        self.isImage = False
        self.listLevel = None
        self.listStack = [doc]

        self.clean_paragraphs = clean_paragraphs


    def flushRun(self):
        if self.block is None:
            self.block = document.Paragraph()

        if self.isImage:
            self.block.content.append(
                document.Image(self.propStack[-1].copy(),
                               [b"".join(self.run)]))
            self.isImage = False
        else:
            self.block.content.append(
                document.Text(self.propStack[-1].copy(),
                              [u"".join(self.run)]))

        self.run[:] = []


    def cleanParagraph(self):
        """
        Compress text runs, remove whitespace at start and end,
        skip empty blocks, etc
        """

        runs = self.block.content

        if not runs:
            self.block = None
            return

        if not self.clean_paragraphs:
            return

        joinedRuns = []
        hasContent = False

        for run in runs:

            if run.content[0]:
                hasContent = True
            else:
                continue

            # For whitespace-only groups, remove any property stuff,
            # to avoid extra markup in output
            if not run.content[0].strip():
                run.properties = {}

            # Join runs only if their properties match
            if joinedRuns and (run.properties == joinedRuns[-1].properties):
                joinedRuns[-1].content[0] += run.content[0]
            else:
                joinedRuns.append(run)

        if hasContent:
            # Strip beginning of paragraph
            joinedRuns[0].content[0] = joinedRuns[0].content[0].lstrip()
            # And then strip the end
            joinedRuns[-1].content[0] = joinedRuns[-1].content[0].rstrip()
            self.block.content = joinedRuns
        else:
            self.block = None


    def flushParagraph(self):
        self.flushRun()
        if self.block.content:
            self.cleanParagraph()
            if self.block is not None:
                self.listStack[-1].append(self.block)


    def handle_unicode(self, bit):
        # text strings in Python 2
        self.run.append(bit)


    def handle_str(self, bit):
        # text strings in Python 3
        self.run.append(bit)


    def handle_Push(self, _):
        self.propStack.append(self.propStack[-1].copy())


    def handle_Pop(self, _):
        self.flushRun()
        self.propStack.pop()


    def handle_Para(self, para):

        self.flushParagraph()

        prevListLevel = self.listLevel
        self.listLevel = para.listLevel

        if prevListLevel is not None and self.listLevel is not None:
            if self.listLevel > prevListLevel:
                l = document.List()
                self.listStack.append(l)

            elif self.listLevel < prevListLevel:
                l = self.listStack.pop()
                self.listStack[-1].append(l)

        self.block = None

    def handle_Pict(self, pict):
        self.flushRun()
        self.isImage = True

    def handle_Reset(self, _):
        self.flushRun()
        self.propStack[-1].clear()


    def handle_ReadableMarker(self, marker):
        self.flushRun()
        if marker.val:
            # RTF needs underline markers for hyperlinks,
            # but nothing else does. If we're in a hyperlink,
            # ignore underlines.
            if 'url' in self.propStack[-1] and marker.name == 'underline':
                return

            self.propStack[-1][marker.name] = marker.val
        else:
            if marker.name in self.propStack[-1]:
                del self.propStack[-1][marker.name]

    def handle_ImageMarker(self, marker):
        if marker.val:
            self.propStack[-1][marker.name] = marker.val
        else:
            if marker.name in self.propStack[-1]:
                # Is there any toggle that is applied to images?
                del self.propStack[-1][marker.name]
            else:
                self.propStack[-1][marker.name] = True



class Group(object):

    def __init__(self, reader, parent=None, charsetTable=None):
        self.reader = reader
        self.parent = parent

        if parent:
            self.props = parent.props.copy()
            self.charset = self.parent.charset
        else:
            self.props = {}
            self.charset = self.reader.charset
        self.isPcData = False

        self.specialMeaning = None
        self.skip = False
        self.url = None
        self.image = None
        self.currentParaTag = None
        self.destination = False

        self.charsetTable = charsetTable

        self.content = []
        self.charBuffer = []
        self.skipCount = 0


    def flushChars(self):
        chars = b"".join(self.charBuffer).decode(self.charset, self.reader.errors)
        self.content.append(chars)
        self.charBuffer = []


    def handle(self, control, digits):
        if self.charBuffer and control != b"ansi_escape":  # end of a text range
            self.flushChars()

        if control == b'*':
            self.destination = True
            return

        if self.image and control in [b'emfblip', b'pngblip', b'jpegblip', b'macpict', b'pmmetafile', b'wmetafile',
                                      b'dibitmap', b'wbitmap', b'wbmbitspixel', b'wbmplanes', b'wbmwidthbytes',
                                      b'picw', b'pich', b'picwgoal', b'pichgoal', b'picscalex', b'picscaley',
                                      b'picscaled', b'piccropt', b'piccropb', b'piccropr', b'piccropl', b'picbmp',
                                      b'picbpp', b'bin', b'blipupi', b'blipuid', b'bliptag', b'wbitmap']:
            self.content.append(ImageMarker(control, digits))
            return

        handler = getattr(self, 'handle_%s' % control.decode('ascii'), None)
        if handler is None:
            return

        if digits:
            handler(digits)
        else:
            handler()


    def char(self, byte):
        if self.skipCount:
            self.skipCount -= 1
        else:
            self.charBuffer.append(byte)


    def _finalize(self):

        if self.destination:
            self.skip = True

        if self.specialMeaning is not None:
            self.skip = True

        if self.skip:
            return

        self.flushChars()


    # This is only the default,
    # and is overridden by some controls
    finalize = _finalize


    def flatten(self):
        if self.skip:
            return []

        stuff = [Push]
        for thing in self.content:
            if isinstance(thing, Group):
                stuff.extend(thing.flatten())
            else:
                stuff.append(thing)
        stuff.append(Pop)

        return stuff


    # Header stuff
    def handle_ansi(self): self.charset = self.reader.charset = 'cp1252'
    def handle_mac(self): self.charset = self.reader.charset = 'mac-roman'
    def handle_pc(self): self.charset = self.reader.charset = 'cp437'
    def handle_pca(self): self.charset = self.reader.charset = 'cp850'

    def handle_ansicpg(self, codepage):
        codepage = int(codepage)
        if codepage in _CODEPAGES_BY_NUMBER:
            self.charset = self.reader.charset = _CODEPAGES_BY_NUMBER[codepage]
        else:
            raise ValueError("Unknown codepage %s" % codepage)


    def handle_fonttbl(self):
        self.specialMeaning = 'FONT_TABLE'
        self.charsetTable = {}

    def handle_falt(self):
        self.specialMeaning = 'FONT_ALT_NAME'
        self.isPcData = True

    def handle_fname(self):
        self.specialMeaning = 'NON_TAGGED_NAME'
        self.isPcData = True

    def _setFontCharset(self, charset=None):
        if charset is None:
            charset = self.reader.charset
        # XXX Todo: Figure out a more graceful way to handle the fact that
        # RTF font declarations can be in their own groups or not
        if self.parent.charsetTable is not None:
            self.parent.charsetTable[self.fontNum] = charset
        else:
            self.charsetTable[self.fontNum] = charset

    def handle_f(self, fontNum):
        if 'FONT_TABLE' in (self.parent.specialMeaning, self.specialMeaning):
            self.fontNum = int(fontNum)
            self._setFontCharset()
            self.isPcData = True
        elif self.charsetTable is not None:
            try:
                self.charset = self.charsetTable[int(fontNum)]
            except KeyError:
                # fontNum not found in charsetTable, ignore if requested
                if self.reader.errors == 'ignore':
                    pass
                else:
                    raise

    def handle_fcharset(self, charsetNum):
        if 'FONT_TABLE' in (self.parent.specialMeaning, self.specialMeaning):
            # Theoretically, \fN should always be before \fcharsetN
            # I don't really expect that will always be true, but let's crash
            # if it's not, and see if it happens in the real world.
            charset = _CODEPAGES.get(int(charsetNum))

            if charset is None:
                raise ValueError("Unsupported charset %s" % charsetNum)
            self._setFontCharset(charset)


    def handle_ansi_escape(self, code):
        code = int(code, 16)

        if isinstance(self.charset, dict):
            uni_code = self.charset.get(code)
            if uni_code is None:
                char = u'?'
            else:
                char = six.unichr(uni_code)

        else:
            char = six.int2byte(code)
            if not self.isPcData:
                self.char(char)
                return

        self.content.append(char)


    def handle_control_symbol(self, symbol):
        # Ignore ~, -, and _, since they are optional crap.
        assert len(symbol) == 1
        if symbol in b'\\{}':
            #self.content.append(six.text_type(symbol))
            self.content.append(symbol.decode("ascii"))


    def handle_u(self, codepoint):
        codepoint = int(codepoint)
        try:
            char = six.unichr(codepoint % 2**16)
        except ValueError:
            if self.reader.errors == b'replace':
                char = b'?'
            else:
                raise

        self.content.append(char)
        self.skipCount = self.props.get(b'unicode_skip', 1)

    def handle_uc(self, skipBytes):
        self.props[b'unicode_skip'] = int(skipBytes)


    def handle_par(self):
        p = Para()
        self.content.append(p)
        self.currentParaTag = p


    def handle_pard(self):
        self.content.append(Reset)


    def handle_plain(self):
        self.content.append(Reset)


    def handle_line(self):
        self.content.append(u"\n")


    def handle_b(self, onOff=None):
        val = onOff in (None, b"", b"1")
        self.content.append(ReadableMarker("bold", val))


    def handle_i(self, onOff=None):
        val = onOff in (None, "", "1")
        self.content.append(ReadableMarker("italic", val))


    def handle_ul(self, onOff=None):
        val = onOff in (None, "", "1")
        self.content.append(ReadableMarker("underline", val))


    def handle_strike(self, onOff=None):
        val = onOff in (None, "", "1")
        self.content.append(ReadableMarker("strike", val))


    def handle_ilvl(self, level):
        if self.currentParaTag is not None:
            self.currentParaTag.listLevel = level
        else:
            # Well, now we're in trouble. But I'm pretty sure this
            # isn't supposed to happen anyway.
            pass


    def handle_up(self, amount=None):
        self.content.append(ReadableMarker("super", True))

    def handle_super(self, amount=None):
        self.content.append(ReadableMarker("super", True))

    #Turns off superscripting or subscripting
    def handle_nosupersub(self, amount=None):
        self.content.append(ReadableMarker("sub", False))
        self.content.append(ReadableMarker("super", False))

    def handle_dn(self, amount=None):
        self.content.append(ReadableMarker("sub", True))

    def handle_sub(self):
        self.content.append(ReadableMarker("sub", True))

    def handle_emdash(self):
        self.content.append(u'\u2014')

    def handle_endash(self):
        self.content.append(u'\u2013')

    def handle_lquote(self):
        self.content.append(u'\u2018')

    def handle_rquote(self):
        self.content.append(u'\u2019')

    def handle_ldblquote(self):
        self.content.append(u'\u201C')

    def handle_rdblquote(self):
        self.content.append(u'\u201D')

    def handle_tab(self):
        self.content.append(u'\t')

    def handle_trowd(self):
        self.content.append(u'\n')

    #Handle the image tag
    def handle_pict(self):
        p = Pict()
        self.content.append(p)
        self.image = p
        #Remove the destination control group of the parent, so that the image is preserved
        self.parent.destination = False

    def handle_field(self):
        def finalize():
            if len(self.content) != 2:
                return u""

            destination, content = self.content

            # The destination isn't allowed to contain any controls,
            # so this should be safe.
            # Except when it isn't, like this:
            # {\field{\*\fldinst {\rtlch\fcs1 \af0 \ltrch\fcs0 \insrsid15420660  PAGE   \\* MERGEFORMAT }}
            try:
                destination = u"".join(destination.content)
            except:
                return u""

            match = re.match(r'HYPERLINK "(.*)"', destination)
            if match:
                content.skip = False
                self.content = [ReadableMarker("url", match.group(1)),
                                content]
            else:
                return u""

        self.finalize = finalize


    def __repr__(self):
        return "G(%s)" % repr(self.content)

    def ignore(self, _=None):
        self.skip = True


    # Header
    handle_filetbl = ignore
    handle_colortbl = ignore
    handle_stylesheet = ignore
    handle_listtable = ignore
    handle_listoverridetable = ignore
    handle_revtbl = ignore

    handle_mmath = ignore

    handle_header = ignore
    handle_footer = ignore
    handle_headerl = ignore
    handle_headerr = ignore
    handle_headerf = ignore
    handle_footerl = ignore
    handle_footerr = ignore
    handle_footerf = ignore


    # Document
    handle_info = ignore
    handle_docfmt = ignore
    handle_pgdsctbl = ignore
    handle_listtext = ignore

    # Revision hacks
    handle_revauthdel = ignore



class ReadableMarker(object):
    def __init__(self, name=None, val=None):
        if name is not None:
            self.name = name
        self.val = val

    def __repr__(self):
        if self.val is None:
            return "!%s!" % self.name
        else:
            return "!%s::%s!" % (self.name, self.val)

class ImageMarker(ReadableMarker):
    pass

class Pict(ImageMarker):
    def __init__(self):
        ImageMarker.__init__(self, "Pict")

    def __repr__(self):
        return "!Image!"

class Para(ReadableMarker):
    listLevel = None

    def __init__(self):
        ReadableMarker.__init__(self, "Para")

    def __repr__(self):
        return "!Para:%s!" % self.listLevel


class Reset(ReadableMarker):
    name = "Reset"

class Push(ReadableMarker):
    name = "Push"

class Pop(ReadableMarker):
    name = "Pop"


# Use singleton objects for the dataless markers:
Reset = Reset()
Push = Push()
Pop = Pop()



from pygov_br.django_apps.base import BaseDataImporter
from pygov_br.django_apps.camara_deputados import models
from pygov_br.camara_deputados import cd


class PartyImporter(BaseDataImporter):

    field_relation = {
        'creation_date': 'dataCriacao',
        'extinction_date': 'dataExtincao',
        'name': 'nomePartido',
        'initials': 'siglaPartido',
    }

    def get_model(self):
        return models.Party

    def clean_initials(self, data):
        queryset = models.Party.objects.filter(initials=data)
        if queryset.exists():
            return data + '*'
        return data

    def get_data(self):
        return cd.deputies.parties()


class PartyBlocImporter(BaseDataImporter):

    field_relation = {
        'id': 'idBloco',
        'creation_date': 'dataCriacaoBloco',
        'extinction_date': 'dataExtincaoBloco',
        'name': 'nomeBloco',
        'initials': 'siglaBloco',
    }

    def get_model(self):
        return models.PartyBloc

    def get_data(self):
        return cd.deputies.parties_bloc()


class PartyBlocMemberImporter(BaseDataImporter):

    field_relation = {
        'party_bloc': 'idBloco',
        'party': 'idPartido',
        'adhesion_date': 'dataAdesaoPartido',
        'shutdown_date': 'dataDesligamentoPartido',
    }

    def get_model(self):
        return models.PartyBlocMember

    def get_data(self):
        return cd.deputies.parties_bloc()

    def save_data(self):
        data = self.get_data()
        for data_row in data:
            members_data = data_row['Partidos']['partido']
            for data in members_data:
                data['idBloco'] = data_row['idBloco']
                self._get_object(data)

    def clean_party_bloc(self, data):
        return models.PartyBloc.objects.get(id=data)

    def clean_party(self, data):
        return models.Party(initials=data)


class ParliamentarySeatImporter(BaseDataImporter):

    field_relation = {
        'name': 'nome',
        'initials': 'sigla',
    }

    def get_model(self):
        return models.ParliamentarySeat

    def get_data(self):
        return cd.deputies.parliamentary_seats()

    def after_save_object(self, obj):
        leaders = cd.deputies.parliamentary_seat_leaders(obj.initials)
        leader_data = leaders.get('lider', None)
        if leader_data:
            leader = models.Deputy.objects.get(pk=leader_data['ideCadastro'])
            leader.parliamentary_seat = obj
            leader.parliamentary_seat_leader = True
            leader.save()
        else:
            representative_id = leaders['representante']['ideCadastro']
            representative = models.Deputy.objects.get(pk=representative_id)
            representative.parliamentary_seat = obj
            representative.save()
        vice_leaders = leaders.get('vice_leader', None)
        if vice_leaders:
            for vice_leader in vice_leaders:
                deputy = models.Deputy.objects.get(
                    pk=vice_leader['ideCadastro'])
                deputy.parliamentary_seat = obj
                deputy.save()


class DeputyImporter(BaseDataImporter):

    field_relation = {
        'id': 'ideCadastro',
        'outbuilding': 'anexo',
        'budget_id': 'codOrcamento',
        'condition': 'condicao',
        'email': 'email',
        'phone': 'fone',
        'cabinet': 'gabinete',
        'parliamentary_id': 'idParlamentar',
        'enrollment_id': 'matricula',
        'name': 'nome',
        'parliamentary_name': 'nomeParlamentar',
        'party': 'partido',
        'gender': 'sexo',
        'region': 'uf',
        'photo_url': 'urlFoto',
    }

    def get_model(self):
        return models.Deputy

    def get_data(self):
        return cd.deputies.all()

    def clean_party(self, data):
        return models.Party.objects.get(initials=data)



# -*- coding: utf-8 -*-
from pygov_br.camara_deputados import cd
import datetime
import responses


@responses.activate
def test_deputy_all():
    xml_response = """
    <deputados>
        <deputado>
            <ideCadastro>74784</ideCadastro>
            <comissoes>
                <titular/>
                <suplente/>
            </comissoes>
        </deputado>
        <deputado>
            <ideCadastro>152610</ideCadastro>
            <comissoes>
                <titular/>
                <suplente/>
            </comissoes>
        </deputado>
    </deputados>
    """
    expected_list = [
        {'ideCadastro': 74784,
         'comissoes': {'titular': None, 'suplente': None}},
        {'ideCadastro': 152610,
         'comissoes': {'titular': None, 'suplente': None}}
    ]
    responses.add(
        responses.GET,
        'http://www.camara.gov.br/SitCamaraWS/Deputados.asmx/ObterDeputados',
        body=xml_response, status=200)
    assert cd.deputies.all() == expected_list
    assert len(responses.calls) == 1


@responses.activate
def test_deputy_all_dict():
    xml_response = """
    <deputados>
        <deputado>
            <ideCadastro>74784</ideCadastro>
            <comissoes>
                <titular/>
                <suplente/>
            </comissoes>
        </deputado>
    </deputados>
    """
    expected_list = [
        {'ideCadastro': 74784,
         'comissoes': {'titular': None, 'suplente': None}},
    ]
    responses.add(
        responses.GET,
        'http://www.camara.gov.br/SitCamaraWS/Deputados.asmx/ObterDeputados',
        body=xml_response, status=200)
    assert cd.deputies.all() == expected_list
    assert len(responses.calls) == 1


@responses.activate
def test_deputy_details():
    xml_response = """
    <Deputados>
        <Deputado>
            <numLegislatura>53</numLegislatura>
            <partidoAtual>
                <sigla>PSD</sigla>
            </partidoAtual>
            <comissoes>
                <comissao>
                    <siglaComissao>CEREFPOL</siglaComissao>
                </comissao>
            </comissoes>
            <cargosComissoes>
                <cargoComissoes>
                    <siglaComissao>MESA</siglaComissao>
                </cargoComissoes>
            </cargosComissoes>
            <periodosExercicio>
                <periodoExercicio>
                    <siglaUFRepresentacao>RN</siglaUFRepresentacao>
                </periodoExercicio>
            </periodosExercicio>
            <historicoLider>
                <itemHistoricoLider>
                    <idHistoricoLider>141428</idHistoricoLider>
                </itemHistoricoLider>
            </historicoLider>
        </Deputado>
        <Deputado>
            <numLegislatura>54</numLegislatura>
            <partidoAtual>
                <sigla>PSD</sigla>
            </partidoAtual>
            <comissoes>
                <comissao>
                    <siglaComissao>CEREFPOL</siglaComissao>
                </comissao>
            </comissoes>
            <cargosComissoes>
                <cargoComissoes>
                    <siglaComissao>MESA</siglaComissao>
                </cargoComissoes>
            </cargosComissoes>
            <periodosExercicio>
                <periodoExercicio>
                    <siglaUFRepresentacao>RN</siglaUFRepresentacao>
                </periodoExercicio>
            </periodosExercicio>
            <historicoLider>
                <itemHistoricoLider>
                    <idHistoricoLider>141428</idHistoricoLider>
                </itemHistoricoLider>
            </historicoLider>
        </Deputado>
    </Deputados>
    """
    expected_list = [
        {'numLegislatura': 53,
         'partidoAtual': {'sigla': 'PSD'},
         'comissoes': {'comissao': {'siglaComissao': 'CEREFPOL'}},
         'cargosComissoes': {'cargoComissoes': {'siglaComissao': 'MESA'}},
         'periodosExercicio': {
             'periodoExercicio': {'siglaUFRepresentacao': 'RN'}},
         'historicoLider': {'itemHistoricoLider': {
             'idHistoricoLider': 141428}}},
        {'numLegislatura': 54,
         'partidoAtual': {'sigla': 'PSD'},
         'comissoes': {'comissao': {'siglaComissao': 'CEREFPOL'}},
         'cargosComissoes': {'cargoComissoes': {'siglaComissao': 'MESA'}},
         'periodosExercicio': {
             'periodoExercicio': {'siglaUFRepresentacao': 'RN'}},
         'historicoLider': {'itemHistoricoLider': {
             'idHistoricoLider': 141428}}},
    ]
    responses.add(
        responses.GET,
        'http://www.camara.gov.br/SitCamaraWS/Deputados.asmx/'
        'ObterDetalhesDeputado',
        body=xml_response, status=200)
    assert cd.deputies.details(141428) == expected_list
    assert len(responses.calls) == 1


@responses.activate
def test_deputy_details_dict():
    xml_response = """
    <Deputados>
        <Deputado>
            <numLegislatura>53</numLegislatura>
            <partidoAtual>
                <sigla>PSD</sigla>
            </partidoAtual>
            <comissoes>
                <comissao>
                    <siglaComissao>CEREFPOL</siglaComissao>
                </comissao>
            </comissoes>
            <cargosComissoes>
                <cargoComissoes>
                    <siglaComissao>MESA</siglaComissao>
                </cargoComissoes>
            </cargosComissoes>
            <periodosExercicio>
                <periodoExercicio>
                    <siglaUFRepresentacao>RN</siglaUFRepresentacao>
                </periodoExercicio>
            </periodosExercicio>
            <historicoLider>
                <itemHistoricoLider>
                    <idHistoricoLider>141428</idHistoricoLider>
                </itemHistoricoLider>
            </historicoLider>
        </Deputado>
    </Deputados>
    """
    expected_list = [
        {'numLegislatura': 53,
         'partidoAtual': {'sigla': 'PSD'},
         'comissoes': {'comissao': {'siglaComissao': 'CEREFPOL'}},
         'cargosComissoes': {'cargoComissoes': {'siglaComissao': 'MESA'}},
         'periodosExercicio': {
             'periodoExercicio': {'siglaUFRepresentacao': 'RN'}},
         'historicoLider': {'itemHistoricoLider': {
             'idHistoricoLider': 141428}}},
    ]
    responses.add(
        responses.GET,
        'http://www.camara.gov.br/SitCamaraWS/Deputados.asmx/'
        'ObterDetalhesDeputado',
        body=xml_response, status=200)
    assert cd.deputies.details(141428) == expected_list
    assert len(responses.calls) == 1


@responses.activate
def test_deputy_parties():
    xml_response = """
    <partidos>
        <partido>
            <idPartido>ADB</idPartido>
        </partido>
        <partido>
            <idPartido>AIB</idPartido>
        </partido>
    </partidos>
    """
    expected_list = [{'idPartido': 'ADB'}, {'idPartido': 'AIB'}]
    responses.add(
        responses.GET,
        'http://www.camara.gov.br/SitCamaraWS/Deputados.asmx/ObterPartidosCD',
        body=xml_response, status=200)
    assert cd.deputies.parties() == expected_list
    assert len(responses.calls) == 1


@responses.activate
def test_deputy_parties_dict():
    xml_response = """
    <partidos>
        <partido>
            <idPartido>ADB</idPartido>
        </partido>
    </partidos>
    """
    expected_list = [{'idPartido': 'ADB'}]
    responses.add(
        responses.GET,
        'http://www.camara.gov.br/SitCamaraWS/Deputados.asmx/ObterPartidosCD',
        body=xml_response, status=200)
    assert cd.deputies.parties() == expected_list
    assert len(responses.calls) == 1


@responses.activate
def test_deputy_parties_bloc():
    xml_response = """
    <blocos>
        <bloco>
            <nomeBloco>PV, PPS</nomeBloco>
            <Partidos>
                <partido>
                    <idPartido>PPS</idPartido>
                </partido>
                <partido>
                    <idPartido>PV</idPartido>
                </partido>
            </Partidos>
        </bloco>
    </blocos>
    """
    expected_list = [{
        'nomeBloco': 'PV, PPS',
        'Partidos': {'partido': [{'idPartido': 'PPS'}, {'idPartido': 'PV'}]}
    }]
    responses.add(
        responses.GET,
        'http://www.camara.gov.br/SitCamaraWS/Deputados.asmx/'
        'ObterPartidosBlocoCD',
        body=xml_response, status=200)
    assert cd.deputies.parties_bloc() == expected_list
    assert len(responses.calls) == 1


@responses.activate
def test_deputy_parties_bloc_dict():
    xml_response = """
    <blocos>
        <bloco>
            <nomeBloco>PV, PPS</nomeBloco>
            <Partidos>
                <partido>
                    <idPartido>PPS</idPartido>
                </partido>
            </Partidos>
        </bloco>
        <bloco>
            <nomeBloco>PV, PPS</nomeBloco>
            <Partidos>
                <partido>
                    <idPartido>PPS</idPartido>
                </partido>
            </Partidos>
        </bloco>
    </blocos>
    """
    expected_list = [
        {'nomeBloco': 'PV, PPS',
         'Partidos': {'partido': {'idPartido': 'PPS'}}},
        {'nomeBloco': 'PV, PPS',
         'Partidos': {'partido': {'idPartido': 'PPS'}}}
    ]
    responses.add(
        responses.GET,
        'http://www.camara.gov.br/SitCamaraWS/Deputados.asmx/'
        'ObterPartidosBlocoCD',
        body=xml_response, status=200)
    assert cd.deputies.parties_bloc() == expected_list
    assert len(responses.calls) == 1


@responses.activate
def test_deputy_parliamentary_seats():
    xml_response = """
    <bancadas>
        <bancada sigla="Bloco PP, PTB" nome="Bloco Parlamentar PP, PTB">
            <lider>
                <ideCadastro>160527</ideCadastro>
            </lider>
            <vice_lider>
                <ideCadastro>73666</ideCadastro>
            </vice_lider>
        </bancada>
    </bancadas>
    """
    expected_list = [{'nome': 'Bloco Parlamentar PP, PTB',
                      'sigla': 'Bloco PP, PTB'}]
    responses.add(
        responses.GET,
        'http://www.camara.gov.br/SitCamaraWS/Deputados.asmx/'
        'ObterLideresBancadas',
        body=xml_response, status=200)
    assert cd.deputies.parliamentary_seats() == expected_list
    assert len(responses.calls) == 1


@responses.activate
def test_deputy_parliamentary_seat_leaders():
    xml_response = """
    <bancadas>
        <bancada sigla="Bloco PP, PTB" nome="Bloco Parlamentar PP, PTB">
            <lider>
                <ideCadastro>160527</ideCadastro>
            </lider>
            <vice_lider>
                <ideCadastro>73666</ideCadastro>
            </vice_lider>
            <vice_lider>
                <ideCadastro>626651</ideCadastro>
            </vice_lider>
        </bancada>
        <bancada sigla="Bloco PP" nome="Bloco Parlamentar PP">
            <lider>
                <ideCadastro>160527</ideCadastro>
            </lider>
            <vice_lider>
                <ideCadastro>73666</ideCadastro>
            </vice_lider>
            <vice_lider>
                <ideCadastro>626651</ideCadastro>
            </vice_lider>
        </bancada>
    </bancadas>
    """
    expected_dict = {
        'lider': {'ideCadastro': 160527},
        'vice_lider': [{'ideCadastro': 73666}, {'ideCadastro': 626651}]
    }
    responses.add(
        responses.GET,
        'http://www.camara.gov.br/SitCamaraWS/Deputados.asmx/'
        'ObterLideresBancadas',
        body=xml_response, status=200)
    result_dict = cd.deputies.parliamentary_seat_leaders('Bloco PP, PTB')
    assert result_dict == expected_dict
    assert len(responses.calls) == 1


@responses.activate
def test_deputy_frequency():
    xml_response = """
    <parlamentar>
        <legislatura>54</legislatura>
        <diasDeSessoes2>
            <dia>
                <frequencianoDia>Presença</frequencianoDia>
                <sessoes>
                    <sessao>
                        <frequencia>Presença</frequencia>
                    </sessao>
                </sessoes>
            </dia>
        </diasDeSessoes2>
    </parlamentar>
    """
    expected_list = [{
        'frequencianoDia': u'Presen\xc3\xa7a',
        'sessoes': {'sessao': {'frequencia': u'Presen\xc3\xa7a'}},
    }]
    responses.add(
        responses.GET,
        'http://www.camara.leg.br/sitcamaraws/SessoesReunioes.asmx/'
        'ListarPresencasParlamentar',
        body=xml_response, status=200)
    result_dict = cd.deputies.frequency('10/10/2010', '11/10/2010', 1)
    assert result_dict == expected_list

    result_dict = cd.deputies.frequency(
        datetime.date(2010, 10, 10), datetime.date(2010, 10, 11), 1)
    assert result_dict == expected_list

    result_dict = cd.deputies.frequency(
        datetime.datetime(2010, 10, 10), datetime.datetime(2010, 10, 11), 1)
    assert result_dict == expected_list
    assert len(responses.calls) == 3


@responses.activate
def test_deputy_frequency_dict():
    xml_response = """
    <parlamentar>
        <legislatura>54</legislatura>
        <diasDeSessoes2>
            <dia>
                <frequencianoDia>Presença</frequencianoDia>
                <sessoes>
                    <sessao>
                        <frequencia>Presença</frequencia>
                    </sessao>
                </sessoes>
            </dia>
            <dia>
                <frequencianoDia>Presença</frequencianoDia>
                <sessoes>
                    <sessao>
                        <frequencia>Presença</frequencia>
                    </sessao>
                </sessoes>
            </dia>
        </diasDeSessoes2>
    </parlamentar>
    """
    expected_list = [
        {'frequencianoDia': u'Presen\xc3\xa7a',
         'sessoes': {'sessao': {'frequencia': u'Presen\xc3\xa7a'}}},
        {'frequencianoDia': u'Presen\xc3\xa7a',
         'sessoes': {'sessao': {'frequencia': u'Presen\xc3\xa7a'}}}
    ]
    responses.add(
        responses.GET,
        'http://www.camara.leg.br/sitcamaraws/SessoesReunioes.asmx/'
        'ListarPresencasParlamentar',
        body=xml_response, status=200)
    result_dict = cd.deputies.frequency('10/10/2010', '11/10/2010', 1)
    assert result_dict == expected_list

    assert len(responses.calls) == 1



# -*- coding: utf-8 -*-
from pygov_br.camara_deputados import cd
from pygov_br.exceptions import MissingParameterError
from datetime import datetime, date, time
import responses
import pytest


@responses.activate
def test_proposal_filter():
    xml_response = """
    <proposicoes>
        <proposicao>
            <nome>PL 2718/2011</nome>
            <tipoProposicao>
                <sigla>PL</sigla>
            </tipoProposicao>
            <orgaoNumerador>
                <sigla>PLEN</sigla>
            </orgaoNumerador>
            <regime>
                <txtRegime>Urgência</txtRegime>
            </regime>
            <apreciacao>
                <txtApreciacao>Apreciação do Plenário</txtApreciacao>
            </apreciacao>
            <autor1>
                <txtNomeAutor>João Paulo Cunha</txtNomeAutor>
            </autor1>
            <ultimoDespacho>
                <txtDespacho>
                Sujeita à Apreciação
                </txtDespacho>
            </ultimoDespacho>
            <situacao>
                <descricao>Arquivada</descricao>
                <orgao>
                    <siglaOrgaoEstado>ARQUIVO</siglaOrgaoEstado>
                </orgao>
                <principal>
                    <proposicaoPrincipal>PL 2473/2011</proposicaoPrincipal>
                </principal>
            </situacao>
        </proposicao>
    </proposicoes>
    """
    expected_list = [
        {'nome': 'PL 2718/2011',
         'tipoProposicao': {'sigla': 'PL'},
         'orgaoNumerador': {'sigla': 'PLEN'},
         'regime': {'txtRegime': u'UrgÃªncia'},
         'apreciacao': {'txtApreciacao': u'ApreciaÃ§Ã£o do PlenÃ¡rio'},
         'autor1': {'txtNomeAutor': u'JoÃ£o Paulo Cunha'},
         'ultimoDespacho': {'txtDespacho': u'Sujeita Ã\xa0 ApreciaÃ§Ã£o'},
         'situacao': {'descricao': 'Arquivada',
                      'orgao': {'siglaOrgaoEstado': 'ARQUIVO'},
                      'principal': {'proposicaoPrincipal': 'PL 2473/2011'}}}
    ]
    responses.add(
        responses.GET,
        'http://www.camara.gov.br/SitCamaraWS/Proposicoes.asmx/'
        'ListarProposicoes',
        body=xml_response, status=200)
    assert cd.proposals.filter(proposal_type='PL', year=2010) == expected_list
    result_list = cd.proposals.filter(
        proposal_type='PL',
        year=2010,
        initial_date=datetime(2010, 10, 10),
        final_date=datetime(2010, 10, 10))
    assert result_list == expected_list

    with pytest.raises(ValueError):
        cd.proposals.filter(proposal_type='PL', year=2010, in_progress='3')

    with pytest.raises(ValueError):
        cd.proposals.filter(proposal_type='PL', year=2010, author_gender='x')

    with pytest.raises(MissingParameterError):
        cd.proposals.filter()

    with pytest.raises(MissingParameterError):
        cd.proposals.filter(proposal_type='PL')

    with pytest.raises(MissingParameterError):
        cd.proposals.filter(year=2010)

    assert len(responses.calls) == 2


@responses.activate
def test_proposal_filter_dict():
    xml_response = """
    <proposicoes>
        <proposicao>
            <nome>PL 2718/2011</nome>
            <tipoProposicao>
                <sigla>PL</sigla>
            </tipoProposicao>
            <orgaoNumerador>
                <sigla>PLEN</sigla>
            </orgaoNumerador>
            <regime>
                <txtRegime>Urgência</txtRegime>
            </regime>
            <apreciacao>
                <txtApreciacao>Apreciação do Plenário</txtApreciacao>
            </apreciacao>
            <autor1>
                <txtNomeAutor>João Paulo Cunha</txtNomeAutor>
            </autor1>
            <ultimoDespacho>
                <txtDespacho>
                Sujeita à Apreciação
                </txtDespacho>
            </ultimoDespacho>
            <situacao>
                <descricao>Arquivada</descricao>
                <orgao>
                    <siglaOrgaoEstado>ARQUIVO</siglaOrgaoEstado>
                </orgao>
                <principal>
                    <proposicaoPrincipal>PL 2473/2011</proposicaoPrincipal>
                </principal>
            </situacao>
        </proposicao>
        <proposicao>
            <nome>PL 2718/2011</nome>
            <tipoProposicao>
                <sigla>PL</sigla>
            </tipoProposicao>
            <orgaoNumerador>
                <sigla>PLEN</sigla>
            </orgaoNumerador>
            <regime>
                <txtRegime>Urgência</txtRegime>
            </regime>
            <apreciacao>
                <txtApreciacao>Apreciação do Plenário</txtApreciacao>
            </apreciacao>
            <autor1>
                <txtNomeAutor>João Paulo Cunha</txtNomeAutor>
            </autor1>
            <ultimoDespacho>
                <txtDespacho>
                Sujeita à Apreciação
                </txtDespacho>
            </ultimoDespacho>
            <situacao>
                <descricao>Arquivada</descricao>
                <orgao>
                    <siglaOrgaoEstado>ARQUIVO</siglaOrgaoEstado>
                </orgao>
                <principal>
                    <proposicaoPrincipal>PL 2473/2011</proposicaoPrincipal>
                </principal>
            </situacao>
        </proposicao>
    </proposicoes>
    """
    expected_list = [
        {'nome': 'PL 2718/2011',
         'tipoProposicao': {'sigla': 'PL'},
         'orgaoNumerador': {'sigla': 'PLEN'},
         'regime': {'txtRegime': u'UrgÃªncia'},
         'apreciacao': {'txtApreciacao': u'ApreciaÃ§Ã£o do PlenÃ¡rio'},
         'autor1': {'txtNomeAutor': u'JoÃ£o Paulo Cunha'},
         'ultimoDespacho': {'txtDespacho': u'Sujeita Ã\xa0 ApreciaÃ§Ã£o'},
         'situacao': {'descricao': 'Arquivada',
                      'orgao': {'siglaOrgaoEstado': 'ARQUIVO'},
                      'principal': {'proposicaoPrincipal': 'PL 2473/2011'}}},
        {'nome': 'PL 2718/2011',
         'tipoProposicao': {'sigla': 'PL'},
         'orgaoNumerador': {'sigla': 'PLEN'},
         'regime': {'txtRegime': u'UrgÃªncia'},
         'apreciacao': {'txtApreciacao': u'ApreciaÃ§Ã£o do PlenÃ¡rio'},
         'autor1': {'txtNomeAutor': u'JoÃ£o Paulo Cunha'},
         'ultimoDespacho': {'txtDespacho': u'Sujeita Ã\xa0 ApreciaÃ§Ã£o'},
         'situacao': {'descricao': 'Arquivada',
                      'orgao': {'siglaOrgaoEstado': 'ARQUIVO'},
                      'principal': {'proposicaoPrincipal': 'PL 2473/2011'}}},
    ]
    responses.add(
        responses.GET,
        'http://www.camara.gov.br/SitCamaraWS/Proposicoes.asmx/'
        'ListarProposicoes',
        body=xml_response, status=200)
    assert cd.proposals.filter(proposal_type='PL', year=2010) == expected_list
    assert len(responses.calls) == 1


@responses.activate
def test_proposal_get():
    xml_response = """
    <proposicao tipo="PL " numero="3962" ano="2008">
        <nomeProposicao>PL 3962/2008</nomeProposicao>
        <idProposicao>408406</idProposicao>
        <Autor>Poder Executivo</Autor>
    </proposicao>
    """
    expected_dict = {'nomeProposicao': 'PL 3962/2008',
                     'idProposicao': 408406,
                     'Autor': 'Poder Executivo'}
    responses.add(
        responses.GET,
        'http://www.camara.gov.br/SitCamaraWS/Proposicoes.asmx/'
        'ObterProposicao',
        body=xml_response, status=200)
    assert cd.proposals.get(proposal_number=408406,
                            proposal_type='PL',
                            year=2008) == expected_dict
    assert len(responses.calls) == 1


@responses.activate
def test_proposal_get_by_id():
    xml_response = """
    <proposicao tipo="PL " numero="3962" ano="2008">
        <nomeProposicao>PL 3962/2008</nomeProposicao>
        <idProposicao>408406</idProposicao>
        <Autor>Poder Executivo</Autor>
    </proposicao>
    """
    expected_dict = {'nomeProposicao': 'PL 3962/2008',
                     'idProposicao': 408406,
                     'Autor': 'Poder Executivo'}
    responses.add(
        responses.GET,
        'http://www.camara.gov.br/SitCamaraWS/Proposicoes.asmx/'
        'ObterProposicaoPorID',
        body=xml_response, status=200)
    assert cd.proposals.get_by_id(408406) == expected_dict
    assert len(responses.calls) == 1


@responses.activate
def test_proposals_voting():
    xml_response = """
    <proposicao>
        <Sigla>PL</Sigla>
        <Numero>1992</Numero>
        <Ano>2007</Ano>
        <Votacoes>
            <Votacao Resumo="Aprovada" Data="28/2/2012" Hora="20:26"
                     ObjVotacao="SUBEMENDA SUBSTITUTIVA GLOBAL DE PLENÁRIO"
                     codSessao="4531">
            <orientacaoBancada>
                <bancada Sigla="PT" orientacao="Sim "/>
                <bancada Sigla="PMDB" orientacao="Sim "/>
            </orientacaoBancada>
            <votos>
                <Deputado Nome="Henrique Afonso" ideCadastro="73940"
                          Partido="PV " UF="AC" Voto="Sim "/>
                <Deputado Nome="Carlos Souza" ideCadastro="73934"
                          Partido="PSD " UF="AM" Voto="Sim "/>
            </votos>
            </Votacao>
        </Votacoes>
    </proposicao>
    """
    expected_list = [{
        'Resumo': 'Aprovada',
        'Data': date(2012, 2, 28),
        'Hora': time(20, 26),
        'ObjVotacao': u'SUBEMENDA SUBSTITUTIVA GLOBAL DE PLENÃ\x81RIO',
        'codSessao': 4531,
        'orientacaoBancada': [{'Sigla': 'PT', 'orientacao': 'Sim'},
                              {'Sigla': 'PMDB', 'orientacao': 'Sim'}],
        'votos': [{'Nome': 'Henrique Afonso', 'ideCadastro': 73940,
                   'Partido': 'PV', 'UF': 'AC', 'Voto': 'Sim'},
                  {'Nome': 'Carlos Souza', 'ideCadastro': 73934,
                   'Partido': 'PSD', 'UF': 'AM', 'Voto': 'Sim'}]
    }]
    responses.add(
        responses.GET,
        'http://www.camara.gov.br/SitCamaraWS/Proposicoes.asmx/'
        'ObterVotacaoProposicao',
        body=xml_response, status=200)
    assert cd.proposals.voting(proposal_number=408406,
                               proposal_type='PL',
                               year=2008) == expected_list
    assert len(responses.calls) == 1


@responses.activate
def test_proposal_voted():
    xml_response = """
    <proposicoes>
        <proposicao>
            <nomeProposicao>PEC 3/1999</nomeProposicao>
        </proposicao>
    </proposicoes>
    """
    expected_list = [
        {'nomeProposicao': 'PEC 3/1999'}
    ]
    responses.add(
        responses.GET,
        'http://www.camara.gov.br/SitCamaraWS/Proposicoes.asmx/'
        'ListarProposicoesVotadasEmPlenario',
        body=xml_response, status=200)
    assert cd.proposals.voted(proposal_type='PL',
                              year=2008) == expected_list
    assert len(responses.calls) == 1


@responses.activate
def test_proposal_voted_dict():
    xml_response = """
    <proposicoes>
        <proposicao>
            <nomeProposicao>PEC 3/1999</nomeProposicao>
        </proposicao>
        <proposicao>
            <nomeProposicao>PEC 3/1999</nomeProposicao>
        </proposicao>
    </proposicoes>
    """
    expected_list = [
        {'nomeProposicao': 'PEC 3/1999'},
        {'nomeProposicao': 'PEC 3/1999'}
    ]
    responses.add(
        responses.GET,
        'http://www.camara.gov.br/SitCamaraWS/Proposicoes.asmx/'
        'ListarProposicoesVotadasEmPlenario',
        body=xml_response, status=200)
    assert cd.proposals.voted(proposal_type='PL',
                              year=2008) == expected_list
    assert len(responses.calls) == 1


@responses.activate
def test_proposal_processed_in_period():
    xml_response = """
    <proposicoes>
        <proposicao>
            <tipoProposicao>MPV</tipoProposicao>
        </proposicao>
    </proposicoes>
    """
    expected_list = [{'tipoProposicao': 'MPV'}]
    responses.add(
        responses.GET,
        'http://www.camara.gov.br/SitCamaraWS/Proposicoes.asmx/'
        'ListarProposicoesTramitadasNoPeriodo',
        body=xml_response, status=200)
    assert cd.proposals.processed_in_period(
        initial_date='10/10/2010',
        final_date='11/10/2010') == expected_list
    assert cd.proposals.processed_in_period(
        initial_date=datetime(2010, 10, 10),
        final_date=datetime(2010, 10, 11)) == expected_list
    assert len(responses.calls) == 2


@responses.activate
def test_proposal_processed_in_period_dict():
    xml_response = """
    <proposicoes>
        <proposicao>
            <tipoProposicao>MPV</tipoProposicao>
        </proposicao>
        <proposicao>
            <tipoProposicao>MPV</tipoProposicao>
        </proposicao>
    </proposicoes>
    """
    expected_list = [{'tipoProposicao': 'MPV'}, {'tipoProposicao': 'MPV'}]
    responses.add(
        responses.GET,
        'http://www.camara.gov.br/SitCamaraWS/Proposicoes.asmx/'
        'ListarProposicoesTramitadasNoPeriodo',
        body=xml_response, status=200)
    assert cd.proposals.processed_in_period(
        initial_date='10/10/2010',
        final_date='11/10/2010') == expected_list
    assert len(responses.calls) == 1


@responses.activate
def test_proposal_progress():
    xml_response = """
    <proposicao tipo="PL" numero="3962" ano="2008">
        <idProposicao>408406</idProposicao>
        <ultimaAcao>
            <tramitacao>
                <orgao>MESA</orgao>
            </tramitacao>
        </ultimaAcao>
        <andamento>
            <tramitacao>
                <orgao>MESA</orgao>
            </tramitacao>
            <tramitacao>
                <orgao>MESA</orgao>
            </tramitacao>
        </andamento>
    </proposicao>
    """
    expected_dict = {
        'idProposicao': 408406,
        'ultimaAcao': {'tramitacao': {'orgao': 'MESA'}},
        'andamento': {'tramitacao': [{'orgao': 'MESA'},
                                     {'orgao': 'MESA'}]}
    }
    responses.add(
        responses.GET,
        'http://www.camara.leg.br/SitCamaraWS/Orgaos.asmx/'
        'ObterAndamento',
        body=xml_response, status=200)
    assert cd.proposals.progress(proposal_number=1293,
                                 year=2008) == expected_dict
    assert cd.proposals.progress(proposal_number=1293,
                                 initial_date='10/10/2010',
                                 year=2008) == expected_dict
    assert cd.proposals.progress(proposal_number=1293,
                                 initial_date=datetime(2012, 10, 10),
                                 year=2008) == expected_dict
    assert len(responses.calls) == 3


@responses.activate
def test_proposal_amendments():
    xml_response = """
    <Proposicao tipo="PL" Numero="3962" Ano="2008">
        <Substitutivos>
            <Substitutivo CodProposicao="439782"
                          Descricao="SBT 1 CTASP => PL 3962/2008"/>
        </Substitutivos>
        <RedacoesFinais>
            <RedacaoFinal CodProposicao="440524"
                          Descricao="RDF 1 => PL 3962/2008"/>
        </RedacoesFinais>
        <Emendas>
            <Emenda CodProposicao="413968"
                    Descricao="EMC 1/2008 CSSF => PL 3962/2008"/>
            <Emenda CodProposicao="413970"
                    Descricao="EMC 2/2008 CSSF => PL 3962/2008"/>
        </Emendas>
    </Proposicao>
    """
    expected_list = [
        {'CodProposicao': 413968,
         'Descricao': 'EMC 1/2008 CSSF => PL 3962/2008'},
        {'CodProposicao': 413970,
         'Descricao': 'EMC 2/2008 CSSF => PL 3962/2008'},
    ]
    responses.add(
        responses.GET,
        'http://www.camara.leg.br/SitCamaraWS/Orgaos.asmx/'
        'ObterEmendasSubstitutivoRedacaoFinal',
        body=xml_response, status=200)
    assert cd.proposals.amendments(proposal_number=1293,
                                   proposal_type='PL',
                                   year=2008) == expected_list


@responses.activate
def test_proposal_final_wordings():
    xml_response = """
    <Proposicao tipo="PL" Numero="3962" Ano="2008">
        <Substitutivos>
            <Substitutivo CodProposicao="439782"
                          Descricao="SBT 1 CTASP => PL 3962/2008"/>
        </Substitutivos>
        <RedacoesFinais>
            <RedacaoFinal CodProposicao="440524"
                          Descricao="RDF 1 => PL 3962/2008"/>
        </RedacoesFinais>
        <Emendas>
            <Emenda CodProposicao="413968"
                    Descricao="EMC 1/2008 CSSF => PL 3962/2008"/>
            <Emenda CodProposicao="413970"
                    Descricao="EMC 2/2008 CSSF => PL 3962/2008"/>
        </Emendas>
    </Proposicao>
    """
    expected_list = [
        {'CodProposicao': 440524,
         'Descricao': 'RDF 1 => PL 3962/2008'},
    ]
    responses.add(
        responses.GET,
        'http://www.camara.leg.br/SitCamaraWS/Orgaos.asmx/'
        'ObterEmendasSubstitutivoRedacaoFinal',
        body=xml_response, status=200)
    assert cd.proposals.final_wordings(proposal_number=1293,
                                       proposal_type='PL',
                                       year=2008) == expected_list


@responses.activate
def test_proposal_substitutives():
    xml_response = """
    <Proposicao tipo="PL" Numero="3962" Ano="2008">
        <Substitutivos>
            <Substitutivo CodProposicao="439782"
                          Descricao="SBT 1 CTASP => PL 3962/2008"/>
        </Substitutivos>
        <RedacoesFinais>
            <RedacaoFinal CodProposicao="440524"
                          Descricao="RDF 1 => PL 3962/2008"/>
        </RedacoesFinais>
        <Emendas>
            <Emenda CodProposicao="413968"
                    Descricao="EMC 1/2008 CSSF => PL 3962/2008"/>
            <Emenda CodProposicao="413970"
                    Descricao="EMC 2/2008 CSSF => PL 3962/2008"/>
        </Emendas>
    </Proposicao>
    """
    expected_list = [
        {'CodProposicao': 439782,
         'Descricao': 'SBT 1 CTASP => PL 3962/2008'}
    ]
    responses.add(
        responses.GET,
        'http://www.camara.leg.br/SitCamaraWS/Orgaos.asmx/'
        'ObterEmendasSubstitutivoRedacaoFinal',
        body=xml_response, status=200)
    assert cd.proposals.substitutives(proposal_number=1293,
                                      proposal_type='PL',
                                      year=2008) == expected_list


@responses.activate
def test_proposal_comissions_opinion():
    xml_response = """
    <proposicao tipo="PL" numero="3962" ano="2008" codProposicao="408406">
        <integra LinkArquivo="http://www.camara.gov.br/?codteor=595202"/>
        <comissoes>
            <comissao nome="Comissão de Trabalho" sigla="CT" codOrgao="2015">
                <tipoAnalise>Constitucionalidade</tipoAnalise>
                <relator>Arnaldo Faria</relator>
            </comissao>
        </comissoes>
    </proposicao>
    """
    expected_list = [
        {'tipoAnalise': 'Constitucionalidade',
         'relator': 'Arnaldo Faria'}
    ]
    responses.add(
        responses.GET,
        'http://www.camara.leg.br/SitCamaraWS/Orgaos.asmx/'
        'ObterIntegraComissoesRelator',
        body=xml_response, status=200)
    assert cd.proposals.comissions_opinion(proposal_number=1293,
                                           proposal_type='PL',
                                           year=2008) == expected_list


@responses.activate
def test_proposal_comissions_opinion_dict():
    xml_response = """
    <proposicao tipo="PL" numero="3962" ano="2008" codProposicao="408406">
        <integra LinkArquivo="http://www.camara.gov.br/?codteor=595202"/>
        <comissoes>
            <comissao nome="Comissão de Trabalho" sigla="CT" codOrgao="2015">
                <tipoAnalise>Constitucionalidade</tipoAnalise>
                <relator>Arnaldo Faria</relator>
            </comissao>
            <comissao nome="Comissão de Trabalho" sigla="CT" codOrgao="2015">
                <tipoAnalise>Constitucionalidade</tipoAnalise>
                <relator>Arnaldo Faria</relator>
            </comissao>
        </comissoes>
    </proposicao>
    """
    expected_list = [
        {'tipoAnalise': 'Constitucionalidade',
         'relator': 'Arnaldo Faria'},
        {'tipoAnalise': 'Constitucionalidade',
         'relator': 'Arnaldo Faria'}
    ]
    responses.add(
        responses.GET,
        'http://www.camara.leg.br/SitCamaraWS/Orgaos.asmx/'
        'ObterIntegraComissoesRelator',
        body=xml_response, status=200)
    assert cd.proposals.comissions_opinion(proposal_number=1293,
                                           proposal_type='PL',
                                           year=2008) == expected_list


@responses.activate
def test_proposal_types():
    xml_response = """
    <siglas>
        <sigla tipoSigla="ADD " descricao="Adendo" ativa="True"/>
    </siglas>
    """
    expected_list = [
        {'tipoSigla': 'ADD', 'descricao': 'Adendo', 'ativa': True}
    ]
    responses.add(
        responses.GET,
        'http://www.camara.gov.br/SitCamaraWS/Proposicoes.asmx/'
        'ListarSiglasTipoProposicao',
        body=xml_response, status=200)
    assert cd.proposals.types() == expected_list


@responses.activate
def test_proposal_author_types():
    xml_response = """
    <siglas>
        <TipoAutor id="TipoOrgao_11" descricao="Conselho"/>
    </siglas>
    """
    expected_list = [
        {'id': 'TipoOrgao_11', 'descricao': 'Conselho'}
    ]
    responses.add(
        responses.GET,
        'http://www.camara.gov.br/SitCamaraWS/Proposicoes.asmx/'
        'ListarTiposAutores',
        body=xml_response, status=200)
    assert cd.proposals.author_types() == expected_list


@responses.activate
def test_proposal_statuses():
    xml_response = """
    <situacaoProposicao>
        <situacaoProposicao id="1180" descricao="Aguardando Apoiamento"/>
    </situacaoProposicao>
    """
    expected_list = [
        {'id': 1180, 'descricao': 'Aguardando Apoiamento'}
    ]
    responses.add(
        responses.GET,
        'http://www.camara.gov.br/SitCamaraWS/Proposicoes.asmx/'
        'ListarSituacoesProposicao',
        body=xml_response, status=200)
    assert cd.proposals.statuses() == expected_list



# -*- coding: utf-8 -*-
from pygov_br.base import Client
from xml.etree.ElementTree import fromstring, ElementTree
import datetime


class DeputyClient(Client):

    def __init__(self):
        host = 'http://www.camara.gov.br/SitCamaraWS/Deputados.asmx/'
        super(DeputyClient, self).__init__(host)

    def all(self):
        """Fetch all deputies.

        List all deputies acting on Câmara dos Deputados.

        Returns:
            list: A list of deputies. Each deputy is represented as a
            disctionary. For example::

                [{'anexo': 4,
                  'codOrcamento': 2729,
                  'comissoes': {'suplente': None, 'titular': None},
                  'condicao': 'Suplente',
                  'email': 'dep.rosinhadaadefal@camara.gov.br',
                  'fone': '3215-5412',
                  'gabinete': 412,
                  'idParlamentar': 5830905,
                  'ideCadastro': 146949,
                  'matricula': 586,
                  'nome': 'ROSEANE CAVALCANTE DE FREITAS',
                  'nomeParlamentar': 'ROSINHA DA ADEFAL',
                  'partido': 'PTdoB',
                  'sexo': 'feminino',
                  'uf': 'AL',
                  'urlFoto': 'http://www.camara.gov.br/internet/deputado/' \
                             'bandep/146949.jpg'}, ...]
        """
        xml_response = self._get('ObterDeputados')
        dict_response = self._xml_to_dict(xml_response)
        list_response = dict_response['deputados']['deputado']

        if isinstance(list_response, dict):
            list_response = [list_response]

        return self._safe(list_response)

    def details(self, deputy_id, legislature=''):
        """Fetch detailed information about a specific deputy.

        List detailed information about a specific deputy.

        Args:
            deputy_id (int): Deputy identifier.
            lesgislature (int, optional): To specify an legislature.
                Defaults to ''.

        Returns:
            list: A list of dictionaries containing extra information about
            the deputy. Each dictionary represents one legislative period. For
            example::

                [{'cargosComissoes': None,
                  'comissoes': {'comissao': {'condicaoMembro': 'Titular',
                    'dataEntrada': datetime.date(2016, 10, 25),
                    'dataSaida': None,
                    'idOrgaoLegislativoCD': 537480,
                    'nomeComissao': 'Comissão de Defesa dos Direitos das ' \
                                    'Pessoas com Deficiência',
                    'siglaComissao': 'CPD'}},
                  'dataFalecimento': None,
                  'dataNascimento': datetime.date(1973, 4, 22),
                  'email': 'dep.rosinhadaadefal@camara.gov.br',
                  'filiacoesPartidarias': None,
                  'gabinete': {'anexo': 4, 'numero': 412,
                               'telefone': '3215-5412'},
                  'historicoLider': None,
                  'historicoNomeParlamentar': None,
                  'idParlamentarDeprecated': 10007,
                  'ideCadastro': 146949,
                  'nomeCivil': 'ROSEANE CAVALCANTE DE FREITAS',
                  'nomeParlamentarAtual': 'ROSINHA DA ADEFAL',
                  'nomeProfissao': 'Funcionário Público Federal',
                  'numLegislatura': 55,
                  'partidoAtual': {'idPartido': 'PTdoB',
                   'nome': 'Partido Trabalhista do Brasil',
                   'sigla': 'PTdoB'},
                  'periodosExercicio': {'periodoExercicio': {'dataFim': None,
                    'dataInicio': datetime.date(2016, 10, 17),
                    'descricaoCausaFimExercicio': None,
                    'idCadastroParlamentarAnterior': 178843,
                    'idCausaFimExercicio': None,
                    'siglaUFRepresentacao': 'AL',
                    'situacaoExercicio': 'Suplente'}},
                  'sexo': False,
                  'situacaoNaLegislaturaAtual': 'Em Exercício',
                  'ufRepresentacaoAtual': 'AL'}, ...]

        """
        path = 'ObterDetalhesDeputado?ideCadastro={0}&numLegislatura={1}'
        xml_response = self._get(path.format(deputy_id, legislature))
        dict_response = self._xml_to_dict(xml_response)
        list_response = dict_response['Deputados']['Deputado']

        if isinstance(list_response, dict):
            list_response = [list_response]

        return self._safe(list_response)

    def parties(self):
        """Fetch all parties.

        List all parties with representation on Câmara dos Deputados.

        Returns:
            list: A list of dictionaries containing the information about the
            parties. For example::

                [{'dataCriacao': None,
                  'dataExtincao': datetime.date(1950, 1, 1),
                  'idPartido': 'USDH',
                  'nomePartido': 'União Social pelos Direitos do Homem',
                  'siglaPartido': 'USDH'}, ...]

        """
        xml_response = self._get('ObterPartidosCD')
        dict_response = self._xml_to_dict(xml_response)
        list_response = dict_response['partidos']['partido']

        if isinstance(list_response, dict):
            list_response = [list_response]

        return self._safe(list_response)

    def parties_bloc(self, bloc_id='', legislature=''):
        """Fetch all parties bloc.

        List all parties blocs or filtering by legislature and id.

        Args:
            bloc_id (int): Parties bloc identifier. Optional.
            lesgislature (int, optional): To specify an legislature.
                Defaults to ''.

        Returns:
            list: A list of dictionaries containing bloc information and a list
            of parties that belongs to the bloc. For example::

                [{'Partidos': {'partido': [
                     {'dataAdesaoPartido': datetime.date(2011, 2, 1),
                      'dataDesligamentoPartido': datetime.date(2013, 10, 16),
                      'idPartido': 'PHS',
                      'nomePartido': 'Partido Humanista da Solidariedade',
                      'siglaPartido': 'PHS'}, ...]
                  'dataCriacaoBloco': datetime.date(2011, 2, 1),
                  'dataExtincaoBloco': datetime.date(2015, 1, 31),
                  'idBloco': 9,
                  'nomeBloco': 'PR, PTdoB, PRP',
                  'siglaBloco': 'PR, PTdoB, PRP'}, ...]

        """
        path = 'ObterPartidosBlocoCD?numLegislatura={0}&idBloco={1}'
        xml_response = self._get(path.format(legislature, bloc_id))
        dict_response = self._xml_to_dict(xml_response)
        list_response = dict_response['blocos']['bloco']

        if isinstance(list_response, dict):
            list_response = [list_response]

        return self._safe(list_response)

    def parliamentary_seats(self):
        """Fetch all parliamentary seats.

        Returns:
            list: A list of dictionaries containing the information about
            parliamentaries seats. For example::

                [{'nome': 'Partido Social Liberal', 'sigla': 'PSL'}, ...]

        """
        xml_response = self._get('ObterLideresBancadas')
        list_response = self._xml_attributes_to_list(xml_response, 'bancada')
        return self._safe(list_response)

    def parliamentary_seat_leaders(self, seat_initials):
        """Fetch parliamentary seat leaders.

        List all leaders and vice-leaders of a specific paliamentary seat.

        Args:
            seat_initials (str): Parliamentary seat initials identifier.

        Returns:
            dict: A dictionary with two keys: 'lider' and 'vice_lider', where
            'lider' is a dictionary and 'vice_lider' a list of dictionaries.
            Both dictionaries contains deputies informations. For example::

                {'lider': {'ideCadastro': 74558,
                           'nome': 'GIVALDO CARIMBÃO',
                           'partido': 'PHS',
                           'uf': 'AL'},
                 'vice_lider': [{'ideCadastro': 178929,
                                 'nome': 'DIEGO GARCIA',
                                 'partido': 'PHS',
                                 'uf': 'PR'}, ...]}

        """
        xml_response = self._get('ObterLideresBancadas')
        element_tree = ElementTree(fromstring(xml_response))
        seats = element_tree.findall('bancada')

        for seat in seats:
            # There is better ways to do this, but we want to support python2.6
            if seat.attrib.get('sigla') == seat_initials:
                parliamentary_seat = seat
        dict_response = self._make_dict_from_tree(parliamentary_seat)
        return self._safe(dict_response['bancada'])

    def frequency(self, initial_date, final_date, parliamentary_enrollment):
        """Fetch parliamentary frequency.

        List the frequency of a parliamentary in a period.

        Args:
            initial_date (str or datetime): Initial date of period. If `str`,
                must be in the format: `dd/mm/yyyy`.
            final_date (str or datetime): Final date of period. If `str`,
                must be in the format: `dd/mm/yyyy`.
            parliamentary_enrollment (int): Parliamentary enrollment number.

        Returns:
            list: A list of sessions that occurred on the specified period and
            the frequency of parliamentarians in each session. For example::

                [{'data': datetime.date(2012, 11, 20),
                  'qtdeSessoes': 3,
                  'sessoes': {'sessao': [{
                      'descricao': 'ORDINÁRIA Nº 313 - 20/11/2012',
                     'frequencia': 'Presença'}, ...]},
                  'frequencianoDia': 'Presença',
                  'justificativa': None} ...]

        """
        if isinstance(initial_date, (datetime.date, datetime.datetime)):
            initial_date = initial_date.strftime('%d/%m/%Y')
        if isinstance(final_date, (datetime.date, datetime.datetime)):
            final_date = final_date.strftime('%d/%m/%Y')

        path = "ListarPresencasParlamentar?dataIni={0}&dataFim={1}&" \
               "numMatriculaParlamentar={2}"
        xml_response = self._get(
            path.format(initial_date, final_date, parliamentary_enrollment),
            host="http://www.camara.leg.br/sitcamaraws/SessoesReunioes.asmx/"
        )
        dict_response = self._xml_to_dict(xml_response)
        list_response = dict_response['parlamentar']['diasDeSessoes2']['dia']

        if isinstance(list_response, dict):
            list_response = [list_response]

        return self._safe(list_response)
